/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TSamplePQryDL.cpp
 * Description    : Connect to DL Datastore
 *                  Create query string
 *                  specify to retrieve folder and docs.
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  recursively display folder and its document/
 *                  folder content retrieving each document/folder
 *                  as it goes.
 *                  Disconnect from DL Datastore
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKParametricQuery.hpp>
#include <dkCollection.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   //-------------------------------------------------------------- 
   // Checking for input parameters
   //--------------------------------------------------------------   
   if (argc < 3)
   {
       cout<<"Invoke Syntax: "<<endl;
       cout<<"  SamplePQryDL <libsrv> <userid> <pw>" << endl;
       return;
   }
 
   // ### get input parameters
   DKString libsrv = DKString(argv[1]);
   DKString userid = DKString(argv[2]);
   DKString pw = DKString(argv[3]);
   DKDatastoreDL dsDL; // must be outside try/catch block
   try {
         dkQuery* pQry;
         DKAny any;
         DKResults* pResults;

         cout << "connecting to datastore" << endl;
         dsDL.connect(libsrv,userid,pw);
         cout << "datastore connected libsrv: " << libsrv << " userid: " << userid << endl;

         DKString cmd = "SEARCH=(INDEX_CLASS=DLSAMPLE,";
         cmd += "MAX_RESULTS=5,";
         cmd += "COND=(DLSEARCH_DocType <> NULL));";
         cmd += "OPTION=(CONTENT=NO;";
         cmd += "TYPE_QUERY=STATIC;TYPE_FILTER=FOLDERDOC)";
         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsDL.createQuery(cmd);
         cout << "executing query" << endl;
         pQry->execute();
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result(); 
         pResults = (DKResults*)((dkCollection*) any);
         
         processResults(pResults);

         dsDL.disconnect();
         cout << "datastore disconnected" << endl;
         delete pQry;
         delete pResults;
    }
    catch(DKException &exc)
    {
         cout << "Error id " << exc.errorId() << endl;
         for(unsigned long i = 0; i < exc.textCount(); i++) {
             cout << "Error text: " << exc.text(i) << endl;
         }
         for (unsigned long j=0; j < exc.locationCount(); j++) {
              const DKExceptionLocation* p = exc.locationAtIndex(j);
              cout << "Filename: "   << p->fileName()     << endl;
              cout << "Function: "   << p->functionName() << endl;
              cout << "LineNumber: " << p->lineNumber()   << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;
   DKParts* parts;
   DKFolder* folder;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
      item->retrieve();
      itemType = item->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_CM_DOCUMENT:
             cout << " Item is a document, " 
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
                 if (dataName == DKPARTS) {
                      any   = item->getData(i);
                      parts = (DKParts*) any.value();
                      if (parts != NULL) {
                         cout << "\t this document has " << 
                              parts->cardinality()  << " parts " << endl;
                         // iterate here to get parts
                      }
                 }   
             }
             break;
        case DK_CM_FOLDER:
             cout << " Item is a folder " << endl;
             folder = 0;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
                 if (dataName == DKPARTS) {
                      any   = item->getData(i);
                      parts = (DKParts*) any.value();
                      if (parts != NULL) {
                          cout << "\t this folder has " << 
                                 parts->cardinality()  << " parts " << endl;
                          // iterate here to get parts
                      }
                 }   
                 if (dataName == DKFOLDER) {
                     any    = item->getData(i);
                     folder = (DKFolder*) any.value();
                     if (folder != NULL) {
                        cout << "\t this folder has "  << 
                                 folder->cardinality() << 
                                " document/folder " << endl;
                     }
                 }   
             }
             if (folder && folder->cardinality() > 0) { 
                 cout << "     ---------- list of folder contents -----------" << endl;
                 processResults(folder); 
                 cout << "     ----------------- end of list ----------------" << endl;
             }
             else cout << " empty folder " << endl;
             break;
        default:
             cout << " Item is not recognized " << endl;
      }     
   }
   delete iter;
}
