/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolFed.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreFed.hpp>
#include <DKDatastorePool.hpp>
#include <DKDatastoreDefFed.hpp>
#include <DKSequentialCollection.hpp>
#include <DKDatastoreAdminFed.hpp>
#include <DKServerMapDefFed.hpp>
#include <DKServerDefFed.hpp>
#include <DKAny.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>


void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   DKString libsrv = "";         // db - icmnlsdb
   DKString userid = "";         // userid
   DKString pw = "";             // password
   DKString connect_string = ""; // DBAUTH option
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolFed <db> <userid> <pw> <connect string>"<<endl;
     return;
    }
    else
    {
    // ### get input parameters
      libsrv = DKString(argv[1]);
      userid = DKString(argv[2]);
      pw = DKString(argv[3]);
      if (argc > 4)
        connect_string = DKString(argv[3]);
    }

    DKString factoryFedLName = "libcmbfedfac815";

   try {
      DKDatastorePool* dsrFed = new DKDatastorePool("Fed", factoryFedLName);
      dsrFed->setDatastoreName(libsrv);
      dsrFed->setMaxPoolSize(10);
      dsrFed->setMinPoolSize(3);
      dsrFed->setTimeOut(1); 
      if (connect_string == "")
        connect_string = "DBAUTH=SERVER";
      dsrFed->setConnectString(connect_string);

      dsrFed->initConnections(userid, pw, 2);
      //dsrFed->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrFed->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrFed->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrFed->getTimeOut()<<endl;

      dkDatastore *dsFed1, *dsFed2, *dsFed3, *dsFed4;;
      dsFed1 = dsrFed->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsFed1(ds1) =  " << dsFed1 << endl;
      dsFed2 = dsrFed->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsFed2(ds2) =  " << dsFed2 << endl;

      DKDatastoreSession *dsIdFed1, *dsIdFed2, *dsIdFed3, *dsIdFed4;
      dsIdFed1 = dsrFed->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdFed1(dsid1)="<< dsIdFed1->getDatastore()
        <<", sessionId=" << dsIdFed1->getId()<<endl;
      dsIdFed2 = dsrFed->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdFed2(dsid2)="<<dsIdFed2->getDatastore()
        <<", sessionId=" << dsIdFed2->getId()<<endl;

      dsrFed->returnConnection(dsIdFed2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdFed2(dsid2) =  " << 
        dsIdFed2->getDatastore() << ", sessionId=" << dsIdFed2->getId()<<endl;
      dsrFed->returnConnection(dsFed2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsFed2(ds2) =  " << dsFed2 <<endl; 

      dsIdFed3 = dsrFed->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdFed3(ds2)=" <<
        dsIdFed3->getDatastore() << ", sessionId=" << dsIdFed3->getId()<<endl;
      dsIdFed4 = dsrFed->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdFed4(dsid3)=" <<
        dsIdFed4->getDatastore() << ", sessionId=" << dsIdFed4->getId() <<endl;


      dsIdFed4 = dsrFed->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdFed4(dsid4)=" <<
        dsIdFed4->getDatastore() << ", sessionId=" << dsIdFed4->getId()<<endl;

      dsFed1 = dsrFed->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsFed1(ds3) =  " << dsFed1 <<endl;
      dsFed2 = dsrFed->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsFed2(ds4) =  " << dsFed2 <<endl;
 
      dsFed3 = dsrFed->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsFed3(ds5) =  " << dsFed3 <<endl;
      dsFed4 = dsrFed->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsFed4(ds6) =  " << dsFed4 <<endl;


      dsIdFed3 = dsrFed->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdFed3(dsid2)=" <<
      dsIdFed3->getDatastore() << ", sessionId=" << dsIdFed3->getId()<<endl;


/***
      // ****use dsFed2 to do ListCatalog
      cout<<"||**** start: use dsFed2 to do ListCatalog ****||"<<endl;
      DKSequentialCollection *pCol = 0;
      dkIterator *pIter = 0;
      DKServerDefFed *pSV = 0;
      DKString strServerName;
      DKString strRepType;
      DKAny a;
      long i = 0;
      cout << "list servers" << endl;
      a = dsFed2->listDataSources();
      pCol = (DKSequentialCollection*)((dkCollection*)a);
      pIter = pCol->createIterator();
      while (pIter->more() == TRUE)
      {
       i++;
       pSV = (DKServerDefFed*)((void*)(*pIter->next()));
       strServerName = pSV->getName();
       strRepType = pSV->getRepositoryType();
       cout << "Server Name [" << i << "] - " << strServerName << " Server Repository Type - " << strRepType << endl; 
       delete pSV;
      }
      delete pIter;
      delete pCol;
      cout << i << " servers listed" << endl;

      cout << "     user " << dsFed2->userName() << ", dsName " << dsFed2->datastoreName() << endl;

      // get the Def and Admin
      dkDatastoreDef* defFed   = dsFed2->datastoreDef();
      DKDatastoreAdminFed* adm = (DKDatastoreAdminFed*) defFed->datastoreAdmin();
         
      //=======================================================================================================
      // list back-end servers
      cout << "\nList server names " << endl;
      long len;
      DKString* servers = adm->listServerNames(len);
      DKServerMapDefFed* serverdat = NULL;

      for (i = 0; i < len; i++) {
          cout << "server [" <<  i << "] = " << servers[i] << endl;
          serverdat = adm->getServer(servers[i]);
          cout << " DKServerMapDefFed = " << serverdat->getName() << endl;
          cout << "     name: " << serverdat->getName()           << endl;
          cout << "     type: " << serverdat->getServerType()     << endl;
          cout << " connStr : " << serverdat->getConnectString()  << endl;
          cout << "    desc : " << serverdat->getDescription()    << endl;
          cout << " refTime : " << serverdat->getRefreshTime()    << endl;
          delete serverdat;

      }  
      delete [] servers;

      //=======================================================================================================
      // list federated entity and attribute names

      long siz;
	  DKString* names = dsFed2->listEntityNames(len);
	  dkEntityDef* entityFed = NULL;

      for (i = 0; i < len; i++) {
      	    cout << "name[" << i << "] = " << names[i] << endl;
		    entityFed   = defFed->getEntity(names[i]);
		    cout << "\tDescription= " << entityFed->getDescription() << endl;

		    DKString* attrNames   = entityFed->listAttrNames(siz);
		    if (attrNames != NULL) {
			    for (int j=0; j < siz; j++) {	
				    cout << "\tattrName = " << attrNames[j] << endl;
				    dkAttrDef* attr = entityFed->getAttr(attrNames[j]);
				    cout << "\t\tentityName:   " << attr->getEntityName() << endl;
				    cout << "\t\tdsName:       " << attr->datastoreName() << endl;
				    cout << "\t\tdsType:       " << attr->datastoreType() << endl;
				    cout << "\t\ttype:         " << attr->getType() << endl;
				    cout << "\t\tdescription:  " << attr->getDescription() << endl;
				    cout << "\t\tsize:         " << attr->getSize() << endl;
				    cout << "\t\tid:           " << attr->getId() << endl;
				    cout << "\t\tprecision:    " << attr->getPrecision() << endl;
				    cout << "\t\tscale:        " << attr->getScale() << endl;
				    cout << "\t\tmax:          " << attr->getMax() << endl;
				    cout << "\t\tmin:          " << attr->getMin() << endl;
				    cout << "\t\tstringType:   " << attr->getStringType() << endl;

				    if (attr->isNullable())
					    cout << "\t\tisNullable:   true" << endl;
				    else 
					    cout << "\t\tisNullable:   false" << endl;
	
				    if (attr->isUpdatable())
					    cout << "\t\tisUpdatable:  true" << endl;
				    else 
					    cout << "\t\tisUpdatable:  false" << endl;
			
				    if (attr->isQueryable())
					    cout << "\t\tisQueryable:  true" << endl;
				    else 
					    cout << "\t\tisQueryable:  false" << endl;
				    cout << " " << endl;

                    delete attr;
			    }
		    }
            delete [] attrNames;
            delete entityFed;
	    }
        delete [] names;

      cout<<"||**** end: use dsFed2 to do ListCatalog ****||"<<endl;
      // ****end of use dsFed2 to do ListCatalog 

***/
      dsrFed->returnConnection(dsFed2);
      cout<<" #returnConnection, dsFed2(ds4) =  "<< dsFed2 <<endl;
      dsFed4 = dsrFed->getConnection(userid, pw);
      cout<<" #getConnection, dsFed4 =  "<< dsFed4 <<endl;
      cout<<"    dsFed4 == dsFed2 =  " <<(dsFed4 == dsFed2)<<endl;

    //**** you can add codes to use dsFed4 to do other thing here  
    //**** end of use dsFed4 to do other thing

      dsrFed->returnConnection(dsFed1);
      cout<<" #returnConnection, dsFed1(ds3) =  " << dsFed1 <<endl;
      dsrFed->returnConnection(dsFed3);
      cout<<" #returnConnection, dsFed3(ds5) =  " << dsFed3 <<endl;
      dsrFed->returnConnection(dsIdFed4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdFed4(dsid4) =  " <<
        dsIdFed4->getDatastore() << ", sessionId=" << dsIdFed4->getId() <<endl;
     
/***
      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;
      dsFed2 = (DKDatastoreFed*)dsrFed->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsFed2(ds7) =  " << dsFed2<<endl;
***/

      dsFed2 = (DKDatastoreFed*)dsrFed->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsFed2(ds3) =  " << dsFed2<<endl;
      cout<<" #call delete dsrFed..." <<endl;
      delete dsrFed;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}
