/*****************************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/*****************************************************************************************/

// sample connect/execute/disconnect to OD
/*
 * Filename       : TExecuteOD.cpp
 * Description    : Connect to OD Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 5 items.
 *                  Get the result in a resultset cursor.
 *                  Loop thru results.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from OD Datastore.
 *
 *                  configuration string can be
 *                      CC2MIMEURL=(File:///c:\cmbroot\cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEURL=(HTTP://www.my.com/cmbcc2mime.ini);
 *                    or
 *                      CC2MIMEFILE=(c:\cmbroot\cmbcc2mime.ini);
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKDatastoreOD.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobOD.hpp>
#include <DKPidXDOOD.hpp>
#include <DKEnvironment.hpp>

static DKBoolean CompareIgnoreCase(DKString str1, DKString str2);
static DKString ODdate( int month, int day, int year );
static void setAGQueryCriteria(int testcase, DKString libSrv);

static DKString appgrp = "";
static DKString cmd    = "";

static UINT MaxCount = 5;

void main(int argc, char *argv[])
{
  cout << "begin..." << endl;

  // Indicate this is a Console Application on Windows NT/95 for error
  // message code page conversion to OEM code page. If this was a Windows
  // GUI (Graphic User Interface) Application this would not be necessary
  // because DK_CM_SS_WINDOWS is the default.
  DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);

  //--------------------------------------------------------------
  // Checking for input parameters
  //--------------------------------------------------------------
  if (argc < 4)
  {
    cout<<"Usage: "<<endl;
    cout<<"  TExecuteOD <libsrv> <userid> <pw> <connect_string> <configuration string>" << endl;
    return;
  }
  // ### get input parameters
  DKString libsrv = DKString(argv[1]);
  DKString userid = DKString(argv[2]);
  DKString pw = DKString(argv[3]);
  DKString connect_string = "";
  DKString configuration_string = "";
  if (argc > 4 )
    connect_string = argv[4];
  if (argc > 5 )
    configuration_string = argv[5];

  DKDatastoreOD* dsOD; // must be outside try/catch block

  if (configuration_string != "")
  {
    dsOD  = new DKDatastoreOD(configuration_string);
  }
  else
  {
    dsOD  = new DKDatastoreOD();
  }
  cout << "Datastore OD created" << endl;
  cout << "connect_string=>" << connect_string << "<=" << endl;
  cout << "configuration string=>" << configuration_string << "<=" << endl;

  try
  {
    dkResultSetCursor* pCur = 0;

    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    // Select a testcase by changing this number!!!
    int testcase = 10;
    //@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    setAGQueryCriteria(testcase, libsrv);

    DKNVPair parms[4];
    parms[0] = DKNVPair("APPL_GROUP", appgrp);
    parms[1] = DKNVPair("MAX_RESULTS", DKString(MaxCount));
    parms[2] = DKNVPair("CONTENT", DKString("ATTRONLY"));
    parms[3] = DKNVPair( DK_CM_PARM_END, DKAny((long)0) );

    cout << "\nApplication group =>" << appgrp << "<=" << endl;
    cout << "Query string =>" << cmd << "<=\n" << endl;

    cout << "connecting to datastore" << endl;
    dsOD->connect(libsrv,userid,pw,connect_string);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;
    cout << "connect_string=" << connect_string << endl;

    dkDataObjectBase * pDO = NULL;
    dkCollection* pCol = NULL;
    dkIterator* pIter = NULL;

    DKDDO *p = 0;
    DKDDO *pDDO = 0;
    dkXDO *pXDO = 0;

    DKPidXDOOD *pidXDO = 0;
    DKPid *pid = 0;

    DKAny a;
    int i = 0;
    short k = 0;
    short j = 0;
    USHORT val = 0;
    USHORT cnt = 1;
    int numDataItems = 0;
    long pidIdCnt = 0;
    long pidIndex = 0;

    DKString strData = "";
    DKString strDataName = "";
    DKString strPid;

    cout << "executing query" << endl;

    DKTimestamp timeBefore;
    cout << "Before query, time stamp is " << timeBefore.asString(NULL) << endl;

    cout << "executing query" << endl;
    pCur = dsOD->execute(cmd,DK_CM_SQL_QL_TYPE,parms);
    cout << "query executed" << endl;

    DKTimestamp timeAfter;
    cout << "After query, time stamp is " << timeAfter.asString(NULL) << endl;
    cout << "cardinality: " << pCur->cardinality() << endl;
    cout << "datastore executed query" << endl;

    if (pCur == NULL)
      cout << "cursor is null!!!" << endl;

    while (pCur->isValid())
    {
      p = pCur->fetchNext();
      if (p != NULL)
      {
        cout << "\n==========> " << "Item " << cnt << " <=========" << endl;
        numDataItems = p->dataCount();
        pid = (DKPid*)p->getPidObject();
        strPid = pid->pidString();
        cout << "pid string " << strPid << endl;
        cout << "pid id string " << pid->getId() << endl;
        strPid = pid->getIdString();
        cout << "pid idString " << strPid << endl;
        pidIdCnt = pid->getIdStringCount();
        cout << "pid idString cnt " << pidIdCnt << endl;
        strPid = pid->getPrimaryId();
        cout << "pid primary id " << strPid << endl;
        pidIndex = 0;
        strPid = pid->getIdString(pidIndex);
        cout << "pid item id " << strPid << endl;
        k = p->propertyId(DK_CM_PROPERTY_ITEM_TYPE);
        if (k > 0)
        {
          a = p->getProperty(k);
          val = a;
          cout << "******************************" << endl;
          switch (val)
          {
            case DK_CM_DOCUMENT :
              cout << "Item is document " << endl;
              break;
          }
          cout << "******************************" << endl;
        }

        cout << "  Number of Data Items: " << numDataItems << endl;
        for (j = 1; j <= numDataItems; j++)
        {
          a = p->getData(j);
          strDataName = p->getDataName(j);

          cout << "    " << j << ". Attribute Name: " << strDataName << endl;
          cout << "       type: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_TYPE) << endl;
          cout << "       nullable: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE) << endl;

          if (strDataName != DK_CM_DKPARTS         &&
              strDataName != "DKResource"    &&
              strDataName != "DKViews"       &&
              strDataName != "DKLargeObject" &&
              strDataName != "DKPermissions" &&
              strDataName != "DKFixedView"   &&
              strDataName != "DKAnnotations")
          {
            cout << "       attribute id: " << p->getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID) << endl;
          }

          if (a.typeCode() == DKAny::tc_string)
          {
            DKString astring = a;
            cout << "       Attribute Value (string): =>" << astring << "<=" << endl;
          }
          else if (a.typeCode() == DKAny::tc_long /*Integer*/)
          {
            long along = a;
            cout << "       Attribute Value (long): " << along << endl;
          }
          else if (a.typeCode() == DKAny::tc_short)
          {
            short ashort = a;
            cout << "       Attribute Value (short): " << ashort << endl;
          }
          else if (a.typeCode() == DKAny::tc_date)
          {
            DKDate adate = a;
            cout << "      Attribute Value (date): " << adate << endl;
          }
          else if (a.typeCode() == DKAny::tc_time)
          {
            DKTime atime = a;
            cout << "       Attribute Value (time): " << atime << endl;
          }
          else if (a.typeCode() == DKAny::tc_timestamp)
          {
            DKTimestamp atimestamp = a;
            cout << "       Attribute Value (timestamp): " << atimestamp << endl;
          }
          else if (a.typeCode() == DKAny::tc_collection)
          {
            cout << "       Attribute Value is collection" << endl;
            pCol = a;
            pIter = pCol->createIterator();
            i = 0;

            while (pIter->more() == TRUE)
            {
              i++;
              a = *(pIter->next());
            //pDO = (dkDataObjectBase*)(void*)a;
              pDO = a;

              if (pDO->protocol() == DK_CM_XDO)
              {
                cout << "         dkXDO object " << i << " in collection" << endl;
                pXDO = (dkXDO*)pDO;
                pidXDO = (DKPidXDOOD*)pXDO->getPidObject();
                cout << "           XDO pid string: " << pidXDO->pidString() << endl;

                // Retrieve and open instance handler for an XDO
                pXDO->retrieve();
                // pXDO.open();
              }
            }
            if (pIter != 0)
             delete pIter;
          }
          else if (a.typeCode() != DKAny::tc_null)
          {
            cout << "       Attribute Value (non NULL): " << a << endl;

            if (strDataName == "DKResource"    ||
                strDataName == "DKFixedView"   ||
                strDataName == "DKLargeObject")
            {
              pDO = (dkDataObjectBase*)a;

              if (pDO->protocol() == DK_CM_XDO)
              {
                cout << "         dkXDO object " << endl;
                pXDO = (dkXDO*)pDO;
                pidXDO = (DKPidXDOOD*)pXDO->getPidObject();
                cout << "           XDO pid string: " << pidXDO->pidString() << endl;

                // Retrieve and open instance handler for an XDO
                pXDO->retrieve();
                // pXDO->open();
              }
            }
          }
          else cout << "       Attribute Value is NULL" << endl;
        }
        cnt++;
        delete p;
      }
    }

    DKTimestamp timeAfter2;
    cout << "\nAfter results processed " << timeAfter2.asString(NULL) << endl;
    cout << "\nquery results processed" << endl;
    cout << "Number of items retrieved: " << cnt - 1  << endl;
    if (pCur != 0)
      delete pCur;
    dsOD->disconnect();
    cout << "\ndatastore disconnected" << endl;
    delete dsOD;
  }

  catch (DKException &exc)
  {

    cout << "Error id " << exc.errorId() << endl;
    cout << "Exception id " << exc.exceptionId() << endl;
    for (unsigned long i = 0; i< exc.textCount();i++)
    {
      cout << "Error text: " << exc.text(i) << endl;
    }
    for (unsigned long g=0; g< exc.locationCount();g++)
    {
      const DKExceptionLocation* p = exc.locationAtIndex(g);
      cout << "Filename: " << p->fileName() << endl;
      cout << "Function: " << p->functionName() << endl;
      cout << "LineNumber: " << p->lineNumber() << endl;
    }
    cout << "Exception Class Name: " << exc.name() << endl;
  }
  cout << "done ..." << endl;
}

// --------------------------------------------------------------------
// Convert regular month, day, year values to an OnDemand integer date
// Asumming (and hopefully) the year is in the yyyy format
// --------------------------------------------------------------------
static DKString ODdate( int month, int day, int year )
{
  int days[] = {31,28,31,30,31,30,31,31,30,31,30,31};
  int OD_days = 0;

  // Just in case someone entered a 2 digit year
  if ( year < 100 )
  {
     // 21st century
     if (year < 70)
        year += 2000;
     else
        year += 1900;
  }

  // Get the numbers of days since 1/1/1970 to 1/1/yyyy
  OD_days = ( year - 1970 ) * 365;

  // Add days from leap years
  for ( int yr = 1970; yr < year; yr++ )
  {
    if ((((yr) % 4) == 0) && ((((yr) % 100) != 0) || (((yr) % 400) == 0)))
    {
      OD_days++;
    }
  }

  // Add days sicne 1/1 of this year
  for ( int mon = 0; mon < month-1; mon++ )
  {
    OD_days += days[mon];
  }

  // Add the current day
  OD_days += day;

  // Additional Leap year adjustment
  if ((((year) % 4) == 0) && ((((year) % 100) != 0) || (((year) % 400) == 0)) && month > 2)
    OD_days++;

  DKString od_days_str( OD_days );

  return od_days_str;
}

// --------------------------------------------------------------------
// Compare two strings case insensitively
// --------------------------------------------------------------------
static DKBoolean CompareIgnoreCase(DKString str1, DKString str2)
{
#ifdef WIN32
  return (str1.CompareNoCase(str2) == 0);
#else
  return (IString::lowerCase(str1) == IString::lowerCase(str2));
#endif
}

// --------------------------------------------------------------------
// set application group and query SQL
// --------------------------------------------------------------------
static void setAGQueryCriteria(int testcase, DKString libsrv)
{
  int month = 6;
  int day   = 2;
  int year  = 1997;

  DKString  mon_str( month );
  DKString  day_str( day );
  DKString  yr_str( year );
  DKString  date_str;
  date_str = mon_str + "/" + day_str + "/" + yr_str;
  DKString od_date = ODdate( month, day, year );

  // Application Group and SQL string
  // OD/390 v2.1
  if (CompareIgnoreCase(libsrv, "9.91.121.31"))
  {
    switch (testcase)
    {
      default:
      case 1:  // AFP
        appgrp = "INSURE  01 ";
        cmd = "WHERE (POLICY_NUMBER LIKE '324-1443255%')";  //'324-1443255-11' ";
        break;
      case 2:
        appgrp = "CHKSTMN301 ";
        cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
        break;
      case 3:  // line/large object
        appgrp = "DELINQCY02 ";
        cmd = "WHERE ( LOAN_NUMBER LIKE '%' )";
        break;
      case 4:  // pdf
        appgrp = "KEYBANK 01 ";
        cmd = "WHERE ( ACCOUNT_NUMBER LIKE '%' )";
    }
  }
  // OD/390 v7.1
  else if (CompareIgnoreCase(libsrv, "9.91.121.34"))
  {
    switch (testcase)
    {
      default:
      case 1: // AFP data
        appgrp = "Credit Card Statements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2:
        appgrp = "CHKSTMNT";
        cmd = "WHERE ACCOUNT_NUMBER LIKE '%' ";
        break;
      case 3:  // line data
        appgrp = "Delinqcy-AG";
        cmd = "WHERE ( pagenum != 0 )";
        break;
      case 4:  // line data large object
        appgrp = "LOAN LO";
        od_date = ODdate( 10, 1, 1994 );
        cmd = "WHERE ( load_date = " + od_date + " )";
        //od_date = ODdate( 2, 2, 1970 );
        //cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        //cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 5:  // pdf
        appgrp = "PDF landscape";
        cmd = "WHERE ( region LIKE '%' )";
      case 6:  // AFP
        appgrp = "afpdata";
        cmd = "WHERE (TaxYear = 1995)";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv, "gunnar"))
  {
    switch (testcase)
    {
      default:
      case 1: // Large object AFP - gunnar
        appgrp = "OnDemand Publications";
        cmd = "where bookname LIKE '%'";
        break;
    }
  }
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "everest"))
  {
    switch (testcase)
    {
      default:
      case 1:
        // AFP data
        appgrp = "CRD";
        cmd = "WHERE ( name LIKE '%' )";
        break;
      case 2:
        // Large Object line data
        appgrp = "sp-ebcdic-lo";
        //od_date = ODdate( 1, 1, 1990 );
        //od_date2 = ODdate( 12, 31, 2001 );
        //cmd = "WHERE ( rpt_date BETWEEN " + od_date + " AND " + od_date2 + " )";
        od_date = ODdate( 12, 15, 2000 );
        cmd = "WHERE ( rpt_date > 0 )"; //= " + od_date + " )";
        break;
      case 3: // Loan Delinquency Reports - Large Objects
        appgrp = "LOAN LO";
        od_date = ODdate( 2, 2, 1970 );   // 2/2/1970
        cmd  = "WHERE (( load_date > " + od_date + ") AND ";
        cmd += "(bgn_loan_num <= 10000 AND end_loan_num >= 10000) )";
        break;
      case 4:
        // Large Object line data
        appgrp = "eliz-largeline";
        od_date = ODdate( 3, 19, 1999 );
        cmd = "WHERE ( rptdate = " + od_date + " )";
        break;
      case 5:
        // PDF
        appgrp = "43MBPDF";
        cmd = "WHERE ( rtitle LIKE '%' )";
        break;
    }
  }
  // OD/400 v4.5
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "rdr400c"))
  {
    switch (testcase)
    {
      default:
      case 1:
        appgrp = "CHECKSTMTS ";
        cmd = "WHERE ( KY1VAL LIKE '%' )";
        break;
    }
  }
  // OD/400 v5.1
  else if (CompareIgnoreCase(libsrv.subString(1, 7), "rdr400g"))
  {
    switch (testcase)
    {
      default:
      case 1:
        appgrp = "CheckStatements";
        cmd = "WHERE ( name LIKE '%' )";
        break;
    }
  }
  else // can be any server!
  {
    switch (testcase)
    {
      default:
      case 1: // AFP
        // native OD integer date
        od_date = ODdate( month, day, year );
        cout << "The date '" << date_str <<"' equals to the OnDemand integer date '" << od_date<<"'" << endl;

        appgrp = "CRD";
        //cmd = "WHERE ( crd_date = " + od_date + " )";
        cmd = "WHERE ( name LIKE '%')";
        break;
      case 2:  // line
        appgrp = "Balance";
        cmd = "WHERE ( hiercode LIKE '..21...002')"; //'%.....006')";
        break;
    }
  }
}
