/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Affects of Changes in Attributes of a Group:
    When an attribute is added or removed from an attribute group, this will only impact 
    Component Types that are created after the modification.  Existing component types
    will remain unchanged.  The reasoning is explained in the header documentation of
    SAttributeGroupDefCreationICM.

Datastore Definition's Definition vs. Component Type Definition
    Please refer to SAttributeGroupDefRetrievalICM for an explanation of the difference
    between sources of Attribute Group Definitions.  

Detecting Non-Existant Attr. Group Definitions
    When retrieving Attribute Group Definitions, if it does not exist, null is returned.  
    Usage of the null object will result in a NullPointerException.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeGroupDefRetrievalICM.hpp"
#include "SAttributeGroupDefUpdateICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeGroupDefUpdateICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Modifying, Updating, & Attribute Group Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Modifying Attribute Group Definitions in Memory
 *                    Saving / Updating Attribute Group Definitions
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeGroupDefUpdateICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attribute Groups used here must be already defined.  If not, please run
 *                    SAttributeGroupDefCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeGroupDefRetrievalICM.hpp/cpp
 *                    SAttributeGroupDefUpdateICM_launch.cpp
 *                    SAttributeGroupDefUpdateICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeGroupDefUpdateICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                         << endl;
            cout << "  SAttributeGroupDefUpdateICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."          << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================"     << endl;
        cout << "IBM Information Integrator for Content  v" << ver << endl;
        cout << "Sample Program:  SAttributeGroupDefUpdateICM"     << endl;
        cout << "--------------------------------------------"     << endl;
        cout << " Database: " << database                          << endl;
        cout << " UserName: " << userName                          << endl;
        cout << "============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Retrieving an Attribute Group Definition
            //--------------------------------------------------------------
            // Retrieving the "S_address" attribute Group.
            //
            // For more information on Attribute Group Retrieval, please consult
            // the Sample SAttributeGroupDefRetrievalICM.
            cout << "Retrieving Attribute Group 'S_address'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_address"); // Retrieve the definition.

            cout << "Retrieved Attribute Group 'S_address'." << endl;

            //--------------------------------------------------------------
            // Modify an Attribute Group in Memory
            //--------------------------------------------------------------
            // Attribute Groups can have properties changed and attributes
            // added and removed.  Please consult the Programming Reference
            // documentation for a full list and description of all possibilities.
            //
            // When an attribute is added or removed from an attribute group, 
            // this will only impact Component Types that are created after the 
            // modification.  Existing component types will remain unchanged.
            //
            // For more information on Attribute Retrieval, please consult the
            // Sample SAttributeDefinitionRetrievalICM.
            //
            // This example will add two attributes, "First Name" & "Last Name" to
            // the "Address" attribute group.
            cout << "Modifying Attribute Group Definition 'S_address' in Memory..." << endl;
            
                DKAttrDefICM* firstName = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_firstName"); // Retrieve the definition.
                DKAttrDefICM* lastName  = (DKAttrDefICM*) dsDefICM->retrieveAttr("S_lastName"); 

                if (firstName) attrGroupDef->addAttr(firstName);  // Add attributes to attribute group
                if (lastName) attrGroupDef->addAttr(lastName); 
                
            cout << "Modified Attribute Group Definition 'S_address' in Memory." << endl;
            
            //--------------------------------------------------------------
            // Save / Update Attribute Group Definition in Datastore Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's update() method can be called
            // as well as the Datastore Definition's update(attrGroupDef) method.
            //
            // This example will use the Attribute Group's update() method.
            cout << "Saving / Updating Attribute Group Definition 'S_address' in Datastore..." << endl;

                attrGroupDef->update();  // Save / Update Changes in Datastore Definition

            cout << "Updated Attribute Group Definition 'S_address' in Datastore." << endl;

            //-------------------------------------------------------------
            // Print Updated Attribute Group Definition
            //-------------------------------------------------------------
            cout << "Printing the Updated Attribute Group Definition of 'S_address'..." << endl;

                SAttributeGroupDefRetrievalICM::printAttrGroupDef(attrGroupDef); // Use wrapper funciton from the Retrieval Sample.
                
                delete(attrGroupDef);  // Free Memory.  Attr Group Def will delete Attr Definitions within.

            cout << "Printed the Updated Attribute Group Definition of 'S_address'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
