/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

When Deleting Reference Attributes, use it as an Attribute Group.  Deleting Reference 
Attribute Definitions work exactly the same as Attribute Groups.

NOTE:  When retrieving Attribute Group Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Attribute Group Definition because it was not defined.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrGroupDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SReferenceAttrDefDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SReferenceAttrDefDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Reference Attribute Definitions, Insurance Model Reference Attribute
 *                    Definitions, and Other Sample Reference Attribute Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Reference Attribute Definitions
 *                    Deleting Insurance Model Reference Attribute
 *                    Deletes all Reference Attributes Defined in SReferenceAttrDefCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SReferenceAttrDefDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Reference Attributes used here should be already defined.  If not, Sample
 *                    SReferenceAttrDefCreationICM may be run.  Sample will complete whether
 *                    the Reference Attributes exist or not.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SReferenceAttrDefDeletionICM_launch.cpp
 *                    SReferenceAttrDefDeletionICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SReferenceAttrDefDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                          << endl;
            cout << "  SReferenceAttrDefDeletionICM <database> <userName> <password>"  << endl;
            cout << "  *** Some parameters not specified, using defaults..."           << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================="     << endl;
        cout << "IBM Information Integrator for Content   v" << ver << endl;
        cout << "Sample Program:  SReferenceAttrDefDeletionICM"     << endl;
        cout << "---------------------------------------------"     << endl;
        cout << " Database: " << database                           << endl;
        cout << " UserName: " << userName                           << endl;
        cout << "=============================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //--------------------------------------------------------------
            // Retrieve a Reference Attribute (Group)
            //--------------------------------------------------------------
            // Treat the Reference Attribute (Group) as an actual
            // Attribute Group, since it was created as such.
            //
            // Retrieving the "S_autoClaim" Reference Attribute (Group) Definition.
            cout << "Retrieving Reference Atttribute 'S_autoClaim'..." << endl;

                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_autoClaim"); // Retrieve the definition.

            cout << "Retrieved Reference Attribute 'S_autoClaim'." << endl;

            //--------------------------------------------------------------
            // Delete Reference Attribute Definition in Datastore Definition
            //--------------------------------------------------------------
            // The Attribute Group Definition's del() method can be called
            // as well as the Datastore Definition's del(attrGroupDef) method.
            //
            // This example will use the Attribute Group's del() method.
            cout << "Deleting Reference Attribute Definition 'S_autoClaim' in Datastore..." << endl;

                if(attrGroupDef!=NULL){   // if it is not already deleted...
                    attrGroupDef->del();  // Delete Attribute Group Definition in Datastore Definition
                    delete(attrGroupDef); // Free Memory
                }

            cout << "Deleted Reference Attribute Definition 'S_autoClaim' in Datastore." << endl;

            //-------------------------------------------------------------
            // Retrieve & Delete "S_myRef"
            //-------------------------------------------------------------
            // This method uses the Datastore Definition's del(attrGroup) method.
            cout << "Retrieving & Deleting Reference Attribute Definition 'S_myRef'..." << endl;

                attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup("S_myRef"); // Retrieve the definition.

                if(attrGroupDef!=NULL){ // if it is not already deleted...
                    dsDefICM->del(attrGroupDef);  // Delete Attribute Group Definition in Datastore Definition
                    delete(attrGroupDef);         // Free Memory
                }

            cout << "Deleted Reference Attribute Definition 'S_myRef'." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Reference Attribute Definition (Same function as in SAttributeGroupDefDeletionICM)
    * @param dsICM              Connected DKDatastoreICM object.
    * @param referenceAttrName  Reference Attribute Name for the Reference Attribute Definition to Delete.
    **/
    void SReferenceAttrDefDeletionICM::deleteReferenceAttrDef(DKDatastoreICM * dsICM, char * referenceAttrName){
        cout << "Deleting the Reference Attribute Definition '" << referenceAttrName << "'..." << endl;

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKAttrGroupDefICM* attrGroupDef = (DKAttrGroupDefICM*) dsDefICM->retrieveAttrGroup(referenceAttrName); // Retrieve the definition.
            if(attrGroupDef!=NULL){    // if it is not already deleted...
                attrGroupDef->del();   // Delete the Attribute Group Definition
                delete(attrGroupDef);  // Free Memory
            }
            
        cout << "Deleted the Reference Attribute Definition '" << referenceAttrName << "'." << endl;
    }
