/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/******************************************************************************
 * Filename       : LoadFolderDocDL.cpp
 *
 * Description    : This loader is a sample of using C++ class library's API
 * to create folders and load sample data into the Digital Library.  You should
 * run it with the provided input data file "folder.dat".  The physical files
 * are packaged in "samples.jar".  You should use command 
 * "jar -xvf samples.jar" to unpack it.	 
 *
 * reminder: The "samples.jar" is in:
 *           x:\cmbroot\samples\java\dl	- for NT
 *           /usr/lpp/cmb/samples/java/dl - for AIX
 * 
 * To compile:
 *     in NT system, enter:
 *        nmake -f LoadFolderDocDL.mak debug=1  (debug=1 is optional)
 *     The compiled .exe will be in \db_bin subdirectory) 
 *
 *     in AIX system, enter:
 *        make -f LoadFolderDocDL.mak
 * 
 * To run it as default, this loader assumes following are already defined: 
 * 
 *    Index Class Name - "FOLDERDOC" (defined with the key fields in step 4)
 *    text search server(search service name) - "TM"
 *    text search index - "TMINDEX"
 * 
 * Invoke Syntax:
 *
 *    LoadFolderDocDL <inputDataFile> <outputLog> <userid> <password> <libraryServer>
 *    example:  LoadFolderDocDL folder.dat f1.log frnadmin password LIBSRVRN 
 *
 *    where:  
 *      <inputDataFile>: a input control file, see folder.dat for example. 
 *      <outputLog>: the log file to store the process results, it can be used
 *                   for debug purpose, you can DELETE it later.  
 *      <userid>: the userid you use to logon the library server.
 *      <password>: the password to logon the library server.
 *      <libraryServer>: the library server name
 * 
 ******************************************************************************
 * Following is the instruction to customize the loader to run your own data 
 ******************************************************************************
 * This loader can be modified to work with any index class defined with		
 * different attribute types(key fields).  Rules to follow:
 *
 *  1. The attribute name in the inputDataFile(ie. folder.dat) should
 *     exactly match with the index class's attribute name(case sensitive). 
 *     for example: 
 *
 *     index class attribute names     in folder.dat file
 *     ---------------------------   ----------------------
 *      DLSEARCH_Author              DLSEARCH_Author=Janet Hedges
 *      DLSEARCH_Publisher           DLSEARCH_Publisher=IBM
 *
 *     index class with your           your <inputDataFile> 
 *     defined attribute names         (ie. mine.dat)
 *     ---------------------------   ----------------------
 *      Department                   Department=U6M
 *      Manager                      Manager=John Smith
 *
 *  2. In your inputDataFile, the prefix for the parts to be added should be 
 *     "xdo" or "smi".  The "xdo" is for a regular part, "smi" is for the part
 *     to be indexed by the text search manager.
 *
 *  3. In inputDataFile each folder should start with "item= " followed with
 *     "Folder= ", and each document should start with "item= " followed with
 *     "Document= ".  Each item is defined by beginning with "item= " and 
 *     ending with "end= ".  So you should make sure the pairs of "item= " and
 *     "end= " are correctly matched for each folder and each document. 
 *     Please use "EndOfInput" to indicate the end of inputDataFile.
 *
 *
 *  4. Examples of index class's attributes and folder.dat
 *
 *     index class attr. name     character length min - max
 *     ----------------------    ----------------------------
 *      DLSEARCH_Author            1 - 50
 *      DLSEARCH_Publisher         1 - 50
 *      DLSEARCH_DocType           1 - 32
 *      DLSEARCH_Date              1 - 50
 *      DLSEARCH_Title             1 - 100
 *      DLSEARCH_Subject           1 - 100
 *      DLSEARCH_Authors           1 - 100
 *      DLSEARCH_Editors           1 - 50
 *      DLSEARCH_Source            1 - 100
 *      DLSEARCH_Volume            1 - 20
 *      DLSEARCH_Issue             1 - 20
 *
 *    (all above key field names are defined with key field type as
 *     "Variable character" and character type as "Extended alphanumeric", the
 *     character length min - max are shown as above)
 *
 *     the folder.dat file: (shown as following - case sensitive)
 *    ----------------------------------------------------------------------
 *
 *   ############ below is a folder(Vol11)##################################
 *   item=L1
 *   Folder=Vol11
 *   DLSEARCH_DocType=Vol11 Folder
 *   DLSEARCH_Subject=Vol11 contents
 *     ############# below is a document contained in folder(Vol11)#########
 *     item=L2-1
 *     Document=Vol11-1
 *     DLSEARCH_DocType=Vol11 Document1
 *     DLSEARCH_Author=Anonymous
 *     DLSEARCH_Subject=Contents
 *     DLSEARCH_Publisher=IBM
 *     DLSEARCH_Date=1994
 *     DLSEARCH_Source=Software Quarterly
 *     DLSEARCH_Title=Contents of Software Quarterly Volume 1 Issue 1
 *     DLSEARCH_Volume=1
 *     DLSEARCH_Issue=1
 *     xdo=../../java/dl/vol11/contents.htm
 *     xdo=../../java/dl/vol11/contents.txt
 *     xdo=../../java/dl/images/321smt.gif
 *     xdo=../../java/dl/images/sqcloud0.gif
 *     xdo=../../java/dl/images/sqclouda.gif
 *     xdo=../../java/dl/images/choice.gif
 *     smi=../../java/dl/vol11/contents.dat
 *     end= item L2-1
 *     ############# end of the document(Vol11-1) ##########################
 *     ############# below is another document contained in folder(Vol11)###
 *     item=L2-2
 *     Document=Vol11-2
 *     DLSEARCH_DocType=Vol11 Document2
 *     DLSEARCH_Author=Randall C. Kennedy
 *     DLSEARCH_Subject=32-bit operating systems
 *     DLSEARCH_Publisher=IBM
 *     DLSEARCH_Date=1995
 *     DLSEARCH_Source=Software Quarterly
 *     DLSEARCH_Title=Moving to a 32-Bit Operating System: Myth vs. Reality
 *     DLSEARCH_Volume=1
 *     DLSEARCH_Issue=1
 *     xdo=../../java/dl/vol11/32-bit.htm
 *     xdo=../../java/dl/vol11/32-bit.txt
 *     xdo=../../java/dl/images/sqcdromt.gif
 *     xdo=../../java/dl/images/321.gif
 *     xdo=../../java/dl/images/323.gif
 *     smi=../../java/dl/vol11/32-bit.dat
 *     end= item L2-2
 *     ############# end of the document(Vol11-2) ##########################
 *   end=item L1 
 *   ############### end of the folder(Vol11) ##############################
 *   ############ below is a folder(Vol2) ##################################
 *   item=L1
 *   Folder=Vol2
 *   DLSEARCH_DocType=Vol2 Folder
 *   DLSEARCH_Subject=Vol2 contents
 *     ########## below is a folder(Vol21) contained in folder(Vol2) #######
 *     item=L21
 *     Folder=Vol21
 *     DLSEARCH_DocType=Vol21 Folder
 *     DLSEARCH_Subject=Vol21 contents
 *       ########### below is a document contained in folder(Vol21)#########
 *       item=L3-1
 *       Document=Vol21-1
 *       DLSEARCH_DocType=Vol21 Document1
 *       DLSEARCH_Author=Anonymous
 *       DLSEARCH_Publisher=IBM
 *       DLSEARCH_Date=1996
 *       DLSEARCH_Source=Software Quarterly
 *       DLSEARCH_Title=Contents of Software Quarterly Volume 2 Issue 1
 *       DLSEARCH_Volume=2
 *       DLSEARCH_Issue=1
 *       xdo=../../java/dl/vol21/contents.htm
 *       xdo=../../java/dl/vol21/contents.txt
 *       xdo=../../java/dl/images/covert.gif
 *       xdo=../../java/dl/images/sqhdr.gif
 *       xdo=../../java/dl/images/cover.gif
 *       end=item L3-1
 *       ############# end of the document(Vol21-1) ########################
 *     end=item L21
 *     ############### end of the folder(Vol21) ############################
 *   end=item L1 
 *   ################# end of the folder(Vol2) #############################
 *   EndOfInput
 *   ################# end of the data input file ##########################
 *
 *
 *  note: here the prefix "smi" is for the part to be indexed by
 *  the text search manager.  ie. contents.dat
 *
 *  5. Invoke example: (in NT system, with files in the following directories)
 *     x:\cmbroot\samples\cpp\dl\LoadFolderDocDL.exe -- this program
 *     x:\cmbroot\samples\cpp\dl\folder.dat         -- inputDataFile
 *     x:\cmbroot\samples\java\dl\vol11\ *.*        -- input text files are here
 *     x:\cmbroot\samples\java\dl\images\ *.*       -- input image files are here
 *
 *     In x:\cmbroot\samples\cpp\dl] type as:
 *
 *     LoadFolderDocDL folder.dat folder.log frnadmin password LIBSRVRN 
 *
 *     (note: in AIX system path is /usr/lpp/cmb/samples/cpp/dl/  )
 *
 *  6. If physical files are in different directories then file path in
 *     folder.dat should be modified.
 *
 *  7. This loader requires the indexClass("FOLDERDOC") and
 *     searchIndex("TM-TMINDEX") have been defined first.
 *     The "FOLDERDOC" has attribute names defined as step 4. and all
 *     the field types are "variable character".
 *
 *     If you have different index class name, 
 *     text search server(search service name) or text search index,
 *     please modify the following variables:
 *
 *       indexClassName = "FOLDERDOC",  //or other index class name ie. "BROADWAY"
 *       searchEngine = "SM",          //search manager ie. SM or QBIC
 *       searchIndex = "TM-TMINDEX",   //search server-search index name
 *       searchInfo = "ENU",           //ENU for english, FRA for french...
 *
 *  8. If you want to use this loader to load data without part to be indexed	
 *     by text search manager, please comment out the lines indicated by the 
 *     |**** no text search by comment out following line ****| &  
 *     |**** end of comment out for no text search ***********|
 *     In this case, you don't need to define the search index name. 
 * 
 *****************************************************************************/   
#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <string.h>

#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKParametricQuery.hpp>
#include <DKAny.hpp>
#include <DKResults.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <DKDDO.hpp>
#include <dkXDO.hpp>
#include <DKAttrDefDL.hpp>
#include <DKBlobDL.hpp>
#include <dkResultSetCursor.hpp>
#include <DKEnvironment.hpp>
#define  MAX  50
#define  MAXLEN   1024

 DKString indexClassName = "FOLDERDOC", //Digital Library index class name
          searchEngine = "SM",          //search manager ie. SM or QBIC
          searchIndex = "TM-TMINDEX",   //search service name-search index name
          searchInfo = "ENU";           //ENU for english, FRA for french...

 DKString attrName[MAX];
 short    attrId[MAX];
 unsigned short    attrType[MAX];
 DKBoolean  attrNullable[MAX];
 DKBoolean smiSrv = FALSE;              //flag for calling wakeUpService 
 DKDatastoreDL dsDL;
 int attrCount = 0;
 FILE *out; 
 FILE *in; 

 void prepareAttrList();
 void load(DKDDO *ddo);
 int  findContentClass(DKString fullFileName);
 DKBlobDL* createXDO(DKString fileName, int CClass, int partId);
 DKString  prepareLine(DKString fullFileName, int partId);
 unsigned short  translateAttrTypeToDataItemType(short attrType);

 void main(int argc, char *argv[])
 {

    try
    {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message code page conversion to OEM code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);


      DKString tmpString;
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //--------------------------------------------------------------   
      if (argc <6)
      {
       cout<<"Invoke Syntax: "<<endl;
       cout<<"  LoadFolderDocDL <inputDataFile> <outputLogFile> ";
       cout<<"<userid> <password> <libraryServer>"<<endl;
       return;
      } 

      DKString inputFile  = DKString(argv[1]);
      DKString outputFile = DKString(argv[2]);
      DKString userid     = DKString(argv[3]);
      DKString password   = DKString(argv[4]);
      DKString libSrv     = DKString(argv[5]);

      out = fopen(outputFile, "w");
      if (out == NULL)
      {
         cout<<"can not open "<<outputFile<<" for write !"<<endl;
         return;
      } 
      //### for writing to log
      tmpString = "You enter: TLoadSample " + inputFile + " " + outputFile +
                  " " + userid + " " + password + " " + libSrv + "\n";                   
      fputs((char*)tmpString, out);

      in = fopen(inputFile, "r");
      if (in == NULL)
      {
         cout<<"can not open input data file: "<<inputFile<<endl;
         return;
      } 

      //-------------------------------------------------------------- 
      // connecting to data store
      //-------------------------------------------------------------- 
      cout<<"Connecting datastore, please wait..."<<endl;
      dsDL.connect(libSrv, userid, password,"");
      cout<<"datastore connected"<<endl;
      cout<<"Loading "<<inputFile<<",  please wait.... "<<endl;
      DKAny anyOpt = DK_FALSE;
      dsDL.setOption(DK_DL_OPT_WAKEUPSRV, anyOpt);
      
      prepareAttrList();

      //-------------------------------------------------------------- 
      // start to read input data file
      //-------------------------------------------------------------- 
      DKString line;
      char thisLine[MAXLEN];

      do
      {
         if (fgets(thisLine, MAXLEN, in) == NULL)
            break;
         line = thisLine;
         line.strip();
         if ((line.subString(0, 4)) == "item")
         {
           tmpString = "******** Begin Loading folder: " + line + " **********\n";                 
           fputs((char*)tmpString, out);
           DKDDO *ddo = new DKDDO((ushort)DK_CM_DDO_INITIAL_SIZE);
           load(ddo);
           tmpString = "******** Finish Loading folder: " + line + " **********\n";                 
           fputs((char*)tmpString, out);
           delete ddo;
         }
         else if (  (((char*)line)[0] != '#') &&
                     ((line.subString(0, 10)) != "EndOfInput")  )
         {
              cout<<"Folder or Document should begin with line:  item ="<<endl;
              return;
         }
      } while ((line.subString(0, 10)) != "EndOfInput");

      tmpString = "******** Input Read: " + line + " **********\n";
      fputs((char*)tmpString, out);
      fclose(in);
      if (smiSrv)
      {
/**** no text search by comment out following line ****/
      //### for writing to log
        DKString tmpA = "about to call invokeSearchEngine(";
        tmpA = tmpA + searchEngine;
        tmpA = tmpA + "..)...\n"; 
        DKString tmpB= "invokeSearchEngine(";
        tmpB = tmpB + searchEngine;
        tmpB = tmpB + "..) done. \n";
        cout<<tmpA;
        fputs((char*)tmpA, out);

        dsDL.invokeSearchEngine(searchEngine, searchIndex);

        cout<<tmpB;
        fputs((char*)tmpB, out);
/**** end of comment out for no text search ***********/
      } 
      fclose(out);
      cout<<"Finish loading "<< inputFile <<endl;
      dsDL.disconnect();
      cout<<"datastore disconnected. Done...."<<endl;

    } // end of try

    catch(DKException &exc)
    {
      cout << "Error id" << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text:" << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
 }

 //**********************************************************
 // load - load the DDO, a recursive routine
 //**********************************************************
 void  load(DKDDO *ddo)
 {
    ushort itemtype = 0; 
    DKDDO *item[MAX];
    DKBlobDL *xdo[MAX];
    DKString xdoFiles[MAX];
    DKParts *parts;
    DKFolder *folder;
    DKPid pid;
    DKAny any;

    //### for generated notepart file name
    DKString noteName = "notepart.txt";  

    int smiPartId = 1,                  //partId for search indexed part
        notePartId = 2,                 //partId for notepart    
        partId = 3,                     //regular part start with 3
        CClass;
    int i, n, s;
    short linecount = 0,
          itemcount = 0,
          xdocount = 0,
          dataId = 0;

    DKString line,
             name = "",
             value = "",
             tString;

    DKBoolean folderDDO_exist = FALSE;
    char thisLine[MAXLEN];
    //-------------------------------------------------------------- 
    // setup the general property for the ddo & set data to null
    //-------------------------------------------------------------- 
    pid.setObjectType(indexClassName);        
    ddo->setPid(pid);
    ddo->setDatastore(&dsDL);
    for (i = 0; i < attrCount; i++)
    {
       dataId = ddo->addData(attrName[i]);
       ddo->addDataProperty(dataId, DK_CM_PROPERTY_ATTRIBUTE_ID, attrId[i]);
       ddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, attrType[i]);
       ddo->addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE, attrNullable[i]);
       ddo->addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT,
                            DK_CM_ATTRIBUTE_READWRITE); 
       ddo->setNull(dataId);
    }
    //-------------------------------------------------------------- 
    // start to read a Folder
    //-------------------------------------------------------------- 
    do
    {
       fgets(thisLine, MAXLEN, in); 
       line = thisLine;
       line.strip();
       if ((line == "") ||
           (((char*)line)[0] == '#') )
         continue;
       n = line.indexOf("=", 0);
       if (n == 0) continue;
       linecount++;
       name = value = "";
       s = line.length();
       name = line.subString(0, n-1);
       value = line.subString(n+1, s-n);
       name.strip();
       value.strip();
       DKAny anyValue; 
       if (linecount == 1)
       {
          if ( name == "Folder" )
                itemtype = DK_CM_FOLDER;     //folder is 2
          else if (name == "Document")
                itemtype = DK_CM_DOCUMENT;   //document is 1
          else cout<<"Input Error: "<<
               "Second line must start with Folder or Document"<<endl;
       }
       if (value == "") continue;  		   
       if ((name != "xdo") && (name != "smi")) 
       {
          dataId = ddo->dataId(name);
          if (dataId != 0)
          {
            anyValue = value;
            ddo->setData(dataId, anyValue);
            continue;
          }
       }
       if ((itemtype == DK_CM_FOLDER) &&
           (name == "item"))
       {  
         if (!(folderDDO_exist))
         {
           // ### create an folder ddo with an empty folder
           cout<<"about do ddo->add() of folder...."<<endl;
           ddo->addProperty(DK_CM_PROPERTY_ITEM_TYPE, DK_CM_FOLDER);
           folder = new DKFolder;
           dataId = ddo->addData(DKFOLDER);
           ddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, DK_CM_DATAITEM_TYPE_COLLECTION_DDO);
           any = (dkCollection*)(folder); 
           ddo->setData(dataId, any);
           ddo->add(); 
           tString = "ddo->add() of folder successful...\n";
           fputs((char*)tString, out); 
           cout<<tString;
           folderDDO_exist = TRUE;  
         }

         item[itemcount] = new DKDDO((ushort)DK_CM_DDO_INITIAL_SIZE);
         load(item[itemcount]);             
         itemcount++;
         tString = "===***=== Folder itemDDO #" + DKString(itemcount) +
		        " Created ===***===\n";
         fputs((char*)tString, out);         
       }

       if ( (itemtype == DK_CM_DOCUMENT) &&
            ((name == "xdo") || (name == "smi")) && 
	    (value != "") )
       { 
          if (name == "smi")
          {
             CClass = findContentClass(value); 
             xdoFiles[xdocount] = prepareLine(value, smiPartId);
             xdo[xdocount] = createXDO(value, CClass, (short)smiPartId);
  /**** no text search by comment out following line ****/  
             DKSearchEngineInfoDL aSrchEx;
             aSrchEx.setSearchEngine(searchEngine);
             aSrchEx.setSearchIndex(searchIndex);
             aSrchEx.setSearchInfo(searchInfo);
             xdo[xdocount]->setExtension("DKSearchEngineInfoDL", (dkExtension*)&aSrchEx);              
  /**** no text search by comment out following line ****/  
             smiSrv = TRUE;             
          } 
          else
          {
             CClass = findContentClass(value); 
             xdoFiles[xdocount] = prepareLine(value, partId);
             xdo[xdocount] = createXDO(value, CClass, (short)partId);
             partId++;  
          }     
          xdocount++;
       }      
    } while ((line.subString(0, 3)) != "end");

    tString = "  ****************The End Of " + value + " Reached\n";
    fputs((char*)tString, out);

    if ( itemtype == DK_CM_FOLDER)
    {
        // ### add ddo to folder one by one to save memory
        for (i = 0; i < itemcount; i++) 
        {
          folder->addMember(ddo, item[i]);
          folder->removeAllElements();
          delete item[i];
        } 
        tString = " =======No. of Items added to the Folder: " +
			      DKString(itemcount) + "\n";
        fputs((char*)tString, out); 
    }

    if ( itemtype == DK_CM_DOCUMENT )
    {
        // ### set ddo attribute property
        ddo->addProperty(DK_CM_PROPERTY_ITEM_TYPE, DK_CM_DOCUMENT);  

        //------------------------------------------------
        // create the notepart file & set parts collection
        //------------------------------------------------
        FILE *note;
        note = fopen(noteName, "w");  
        parts = new DKParts; 
      
        for (i = 0; i < xdocount; i++)
	{
           //### for writing to notepart file and log
           tString = xdoFiles[i] + "\n";
           fputs((char*)tString, note);   
           any = (dkDataObjectBase*)xdo[i]; 
           parts->addElement(any);
    	}
        fclose(note);

        //### create notepart xdo
        any = (dkDataObjectBase*)
               (createXDO(noteName, DK_DL_CC_ASCII, notePartId));
	parts->addElement(any); 
 
        //### setup ddo
        dataId = ddo->addData(DKPARTS);
        ddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE, 
                              DK_CM_DATAITEM_TYPE_COLLECTION_XDO);
        any = (dkCollection*)(parts);            
        ddo->setData(dataId, any);

        ddo->add();
        tString = "ddo->add() of document successful...\n";
        fputs((char*)tString, out); 
        cout<<tString;
        tString = "added itemId = " + (ddo->getPid()).getId() + "\n";
        fputs((char*)tString, out);
    }     
    return;
 }

 //************************************************************
 // prepareAttrList - prepare the attributes list for each item
 //************************************************************
 void  prepareAttrList()
 {
     DKSequentialCollection  *collection = 0;
     dkIterator *iterator = 0;
     DKAttrDefDL *attrDef = 0; 
     DKAny *pA = 0;
     collection = (DKSequentialCollection*)((dkCollection*)
                   dsDL.listEntityAttrs(indexClassName));
     iterator = collection->createIterator();
     int i = 0;

     //-------------------------------------------------------------- 
     // obtains data for setting up attrbuteList
     //-------------------------------------------------------------- 
     unsigned short sAttrType = 0;
     while (iterator->more() == TRUE)
     {
        pA = iterator->next();
        attrDef = (DKAttrDefDL*) pA->value();        
        attrName[i]      = attrDef->getName();
        sAttrType = attrDef->getType();
        attrType[i] = translateAttrTypeToDataItemType(sAttrType);
        attrId[i]        = attrDef->getId();
        attrNullable[i]  = attrDef->isNullable();
        i++;
        attrCount++;
        delete attrDef;
     }
     delete iterator;
     delete collection;
     DKString tStr = "Index Class: " + indexClassName + " with " + 
                DKString(attrCount) + " Attributes Obtained\n";
     fputs((char*)tStr, out);
     return;
 }

 //**********************************************************
 // findContentClass - find content class from file extension
 //**********************************************************
 int  findContentClass(DKString fullFileName)
 {
     DKString suffix = "";   
     int n, pos, s, CClass;
     n = pos = s = 0;
     s = fullFileName.length();
     n = fullFileName.indexOf(".", 0);
     while (n != 0)
     {
       pos = n;
       n = fullFileName.indexOf(".", n+1);
     }
     if (pos > 0)
       suffix = fullFileName.subString(pos+1, s-pos);
     if (suffix != "")
     {
        if ((suffix == "htm") || 
	    (suffix == "html")) 
                  CClass = DK_DL_CC_HTML;
	    else if (suffix == "gif")     
                  CClass = DK_DL_CC_GIF;
	    else if (suffix == "jpg")
                  CClass = DK_DL_CC_JPG;
	    else if (suffix == "avi")
                  CClass = DK_DL_CC_AVI;
	    else if (suffix == "wav")
                  CClass = DK_DL_CC_WAV;
	    else if (suffix == "mid")
                  CClass = DK_DL_CC_MIDI;
        else if (suffix == "bmp")
                  CClass = DK_DL_CC_WINV3_BMP;
	    else  CClass = DK_DL_CC_ASCII;             
     }
     else
        CClass = DK_DL_CC_ASCII; 
     return CClass;
 } 

 //*****************************************************
 // createXDO - create an xdo
 //*****************************************************
 DKBlobDL* createXDO(DKString fileName, int CClass, int partId)
 {
      DKBlobDL  *axdo = new DKBlobDL(&dsDL);
      DKPidXDODL apid;
      apid.setPartId(partId);
      axdo->setPidObject(&apid);
      axdo->setContentClass(CClass);
      if ((CClass == DK_DL_CC_ASCII) && (partId == 2))
      {
        axdo->setAffiliatedType(DK_DL_NOTE);
      }
      else
        axdo->setAffiliatedType(DK_DL_BASE);
      axdo->setContentFromClientFile(fileName);

      //### for writing to log
      DKString tmpStr = "  ##creat xdo part: " + fileName +
		        ":" + DKString(partId) + "\n";
      fputs((char*)tmpStr, out);
      return axdo;
 } 

 //**********************************************************
 // prepareLine - prepare a line record for the notepart file
 //**********************************************************
 DKString  prepareLine(DKString fullFileName, int partId)
 {
     DKString suffix = "";   
     int m, n, mpos, npos, s;
     m = n = mpos = npos = s = -1;
     s = fullFileName.length();
     m = fullFileName.indexOf('\\');
     while (m != 0)
     {
       mpos = m;
       m = fullFileName.indexOf('\\', m+1);
     }
     n = fullFileName.indexOf('/');
     while (n != 0)
     {
       npos = n;
       n = fullFileName.indexOf('/', n+1);
     }
     if ((npos == -1) && (mpos == -1))
       suffix = fullFileName + ":" + DKString(partId);
     else if (mpos == -1)
          suffix = fullFileName.subString(npos+1, s-npos) + 
                 ":" + DKString(partId);
     else 
          suffix = fullFileName.subString(mpos+1, s-mpos) + 
                 ":" + DKString(partId);
     return suffix; 
 }

  //**********************************************************
  // translateAttrTypeToDataItemType - change attribute type to
  //   ddo dataitem type 
  //**********************************************************
 unsigned short  translateAttrTypeToDataItemType(short attrType)
 {
   unsigned short rAttrType = DK_CM_DATAITEM_TYPE_UNDEFINED;
   switch(attrType)
   {
    case DK_CM_VSTRING:
    case DK_CM_FSTRING:
    case DK_CM_DECIMAL:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_STRING;
    }
    break;
    case DK_CM_SHORT:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_SHORT;
    }
    break;
    case DK_CM_LONG:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_LONG;
    }
    break;
    case DK_CM_DATE:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_DATE;
    }
    break;
    case DK_CM_TIME:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIME;
    }
    break;
    case DK_CM_TIMESTAMP:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIMESTAMP;
    }
    break;
   }
   return rAttrType;
 }
