
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
// sample list catalog data for IP
/*
 * Filename       : TListCatalogIP.cpp
 * Description    : test DKDatastoreIP for listing methods (Catalog or schema info)
 *
 */

#include <iostream.h>
#include <stdio.h>
#include <DKConstant2IP.h>
#include <DKDatastoreIP.hpp>
#include <DKEntityDefIP.hpp>
#include <DKAttrDefIP.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>
#include <DKSequentialCollection.hpp>

void main(int argc, char *argv[])
{
   cout << " *** Test ImagePlus Connector " << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   //--------------------------------------------------------------
   // Checking for input parameters
   //--------------------------------------------------------------
   if (argc < 4)
   {
      cout<<"Usage: "<<endl;
      cout<<"  TListCatalogIP <libsrv> <userid> <pw>" << endl;
      return;
   }

   // ### get input parameters
   DKString libsrv = DKString(argv[1]);  // FAFCICS61
   DKString userid = DKString(argv[2]);  // <anything>
   DKString pw = DKString(argv[3]);      // <anything>
   DKDatastoreIP dsIP;

   try {
      cout << "---Connect to ImagePlus server---" << endl;

      // Version of IP/390 with Annotation support
      DKString s("APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61");
      dsIP.connect(libsrv, userid, pw, s);

      // List entities...
      DKEntityDefIP* docDef = 0;
      DKAttrDefIP* attrDef = 0;

      cout << "---List entities---" << endl;
      DKSequentialCollection* pCol = (DKSequentialCollection*)(dsIP.listEntities());
      dkIterator* pIter = 0;

      if ( pCol == 0 )
      {
         cout << "collection of entities is null!" << endl;
      }
      else
      {

         // Method 1:
         cout << "---List attributes using listAttrNames and getAttr methods---" << endl;

         pIter = pCol->createIterator();
         while (pIter->more())
         {
            docDef = (DKEntityDefIP*)(pIter->next()->value());
            cout << " Document type     : " << docDef->getType() << endl;
            cout << " Document type name: " << docDef->getName() << endl;

            long tmpCount;
            DKString* attrNames;

            // Upon return, tmpCount contains the number of elements in the list.
            attrNames = docDef->listAttrNames(tmpCount);

            for (int i=0
                 ; i<tmpCount
                 ; i++ )
             {
                  cout << "   Attr name before lookup " << attrNames[i] << endl;
                  attrDef = (DKAttrDefIP*)(docDef->getAttr(attrNames[i]));
                  cout << "   Attr name [" << i << "] : " << attrDef->getName() << endl;
                  cout << "   Attr id       : " << attrDef->getId() << endl;
                  cout << "   Entity name   : " << attrDef->getEntityName() << endl;
                  cout << "   Datastore name: " << attrDef->datastoreName() << endl;
                  cout << "   Attr type     : " << attrDef->getType() << endl;
                  cout << "   Attr restrict : " << attrDef->getStringType() << endl;
                  cout << "   Attr min val  : " << attrDef->getMin() << endl;
                  cout << "   Attr max val  : " << attrDef->getMax() << endl;
                  cout << "   Attr display  : " << attrDef->getSize() << endl;
                  cout << "   Attr precision: " << attrDef->getPrecision() << endl;
                  cout << "   Attr scale    : " << attrDef->getScale() << endl;
                  cout << "   Attr update   ? " << attrDef->isUpdatable() << endl;
                  cout << "   Attr nullable ? " << attrDef->isNullable() << endl;
                  cout << "   Attr queryable? " << attrDef->isQueryable() << endl;
                  cout << "" << endl;
                  delete attrDef;
             }  // end for

             delete [] attrNames;

          }  // end while
         delete pIter;

         // Method 2:
         cout << "---List attributes using listEntityAttrs method---" << endl;

         pIter = pCol->createIterator();
         while (pIter->more())
         {
            docDef = (DKEntityDefIP*)(pIter->next()->value());  // iterator returns DKAny*
            cout << " Document type     : " << docDef->getType() << endl;
            cout << " Document type name: " << docDef->getName() << endl;

            DKSequentialCollection* pAttrCol = (DKSequentialCollection*)(dsIP.listEntityAttrs(docDef->getName()));
            if ( pAttrCol == 0 )
            {
               cout << "collection of entity attrs is null for entity "
                    << docDef->getName()
                    << endl;
            }
            else
            {
               int i=0;
               dkIterator* pAttrIter = pAttrCol->createIterator();
               while (pAttrIter->more())
               {
                  i++;
                  attrDef = (DKAttrDefIP*)(pAttrIter->next()->value());   // iterator returns DKAny*
                  cout << "   Attr name [" << i << "] : " << attrDef->getName() << endl;
                  cout << "   Attr id       : " << attrDef->getId() << endl;
                  cout << "   Entity name   : " << attrDef->getEntityName() << endl;
                  cout << "   Datastore name: " << attrDef->datastoreName() << endl;
                  cout << "   Attr type     : " << attrDef->getType() << endl;
                  cout << "   Attr restrict : " << attrDef->getStringType() << endl;
                  cout << "   Attr min val  : " << attrDef->getMin() << endl;
                  cout << "   Attr max val  : " << attrDef->getMax() << endl;
                  cout << "   Attr display  : " << attrDef->getSize() << endl;
                  cout << "   Attr precision: " << attrDef->getPrecision() << endl;
                  cout << "   Attr scale    : " << attrDef->getScale() << endl;
                  cout << "   Attr update   ? " << attrDef->isUpdatable() << endl;
                  cout << "   Attr nullable ? " << attrDef->isNullable() << endl;
                  cout << "   Attr queryable? " << attrDef->isQueryable() << endl;
                  cout << "" << endl;
                  delete attrDef;
               }  // end while
               delete pAttrIter;
            }
            delete pAttrCol;
            delete docDef;
         }  // end while
         delete pIter;
      }
      delete pCol;

      dsIP.disconnect();
      cout << "datastore disconnected" << endl;
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }

   cout << "done ..." << endl;
}

