/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// Class Name  = SampleGuiAttrEditor
// Description = Sample visual attribute editor/viewer bean
// Author      = mimivo
// Date        = 05/08/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusListener;
import java.beans.*;
import java.io.*;
import java.util.Vector;

/*
import  javax.swing.*;
import  javax.swing.border.*;
*/

import javax.swing.*;
import javax.swing.border.*;

//*****************************************************************************
// class/interface definition
//*****************************************************************************
/**
 * Sample GUI bean to prompt for user ID and password. Can be used with CM
 * connection bean to perform the functions.
 *
 * <pre>
 *   standalone properties  timeout, asynchSearch.
 *   source of events       CMBSearchRequestEvent     - to search
 *                          CMBSearchReply
 * </pre>
 */
public class SampleGuiAttrEditor implements Serializable, CMBUtilConstant, ItemRequestListener {

    //*****************************************************************************
    // constructor
    //*****************************************************************************
    /**
    *  Default constructor
    */
    public SampleGuiAttrEditor() {
        listenerVector = new Vector();
        initGUI(null);
        _item = null;
        _action = 0;
    }

    /**
    * Constructor with search template
    * @param item data item object
    */
    public SampleGuiAttrEditor(CMBItem item, int action) {
        listenerVector = new Vector();
        initGUI(item);
        _item = item;
        _action = action;
    }

    /**
    * Destructor,  dispose the window.
    */
    public void finalize() {
        frame.dispose();
    }

    public void onItemRequest(ItemRequestEvent evt) {
        _action = evt.getID();
        CMBItem item = (CMBItem) evt.getData();
        setItem(item);
    }

    //*****************************************************************************
    // Event source: listener registration, unregistration, and notify methods
    //*****************************************************************************
    // CMBDataRequestEvent
    //*****************************************************************************
    /**
     * CMBDataRequestEvent registration method.
     * It ignores a registration request if the listener is already registered.
     * @param listener listener to be registered
     */
    public void addCMBDataRequestListener(CMBDataRequestListener listener) {
        if (!listenerVector.contains(listener))
            listenerVector.addElement(listener);
    }

    /**
     * CMBDataRequestEvent unregistration method.
     * @param listener listener to be unregistered
     */
    public void removeCMBDataRequestListener(CMBDataRequestListener listener) {
        if (listenerVector.contains(listener))
            listenerVector.removeElement(listener);
    }

    public CMBDataRequestEvent getDataRequestEvent() {
        return dataReqEvent;
    }

    //*****************************************************************************
    // property accessors
    //*****************************************************************************
    /**
    * Property getter.
    * @return  window title
    */
    public String getWindowTitle() {
        return frame.getTitle();
    }

    /**
    * Property setter.
    * @param   newValue new window title
    */
    public void setWindowTitle(String newValue) {
        if (newValue != null)
            frame.setTitle(newValue);
    }

    /**
    * Property getter.
    * @return  connect button label
    */
    public String getOkLabel() {
        return okButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new connect button label
    */
    public void setOkLabel(String newValue) {
        if (newValue != null)
            okButton.setText(newValue);
    }

    /**
    * Property getter.
    * @return  cancel button label
    */
    public String getCancelLabel() {
        return cancelButton.getText();
    }

    /**
    * Property setter.
    * @param   newValue new cancel button label
    */
    public void setCancelLabel(String newValue) {
        if (newValue != null)
            cancelButton.setText(newValue);
    }

    public boolean isVisible() {
        return frame.isVisible();
    }

    //*****************************************************************************
    // Private methods
    //*****************************************************************************
    // Serialize transient data
    //*****************************************************************************
    /**
    * @serialData the data is serialized in the following sequence:
    * <pre>
    *    this object
    *    event listeners if the event listener is serializable
    *    a null object   tells the end of serialization
    * </pre>
    */
    private void writeObject(ObjectOutputStream stream) throws IOException {
        stream.defaultWriteObject();

        Vector v = null;
        synchronized (this) {
            if (listenerVector.size() > 0)
                v = (Vector) listenerVector.clone();
        }

        if (v != null) {
            int cnt = v.size();
            CMBSchemaReplyListener listener = null;
            for (int i = 0; i < cnt; ++i) {
                listener = (CMBSchemaReplyListener) v.elementAt(i);
                if (listener instanceof Serializable)
                    stream.writeObject(listener);
            }
        }

        stream.writeObject(null); // tells the end of listener
    }

    //*****************************************************************************
    // De-serialize/initialize transient data
    //*****************************************************************************
    /**
    * @serialData the data is deserialized in the following sequence:
    * <pre>
    *    this object
    *    event listeners may be none
    *    a null object   the marker of serialization
    * </pre>
    */
    private void readObject(ObjectInputStream stream) throws Exception {
        listenerVector = new Vector(); // always initialize listener list
        stream.defaultReadObject();

        Object o;
        while ((o = stream.readObject()) != null)
            listenerVector.addElement((CMBSchemaReplyListener) o);
        frame.setVisible(true);
    }

    public void setItem(CMBItem item) {
        System.out.println("SampleGuiAttrEditor:setItem()....");
        _item = item;

        _getItemData(item);

        fieldPane.removeAll();
        _setupAttrPane();

        System.out.println("About to show the Attribute Editor main frame ....");
        frame.setVisible(true);
        frame.repaint();
        frame.requestFocus();
    }

    private void _setupAttrPane() {
        fieldPane.setVisible(false);
        int nAttr = attrList.size();

        fieldPane.setLayout(new GridLayout(nAttr, 2, 4, 4));

        txtFieldList.removeAllElements();

        for (int i = 0; i < nAttr; i++) {
            String attrName = (String) attrList.elementAt(i);
            String attrValue = (String) valList.elementAt(i);
            JLabel attrLabel = new JLabel(attrName);
            JTextField attrText = new JTextField(attrValue);

            txtFieldList.addElement(attrText);

            TextFieldActionListener listnr = new TextFieldActionListener(attrName);
            attrText.addActionListener(listnr);

            fieldPane.add(attrLabel);
            fieldPane.add(attrText);
        }
        fieldPane.setVisible(true);
        fieldPane.doLayout();
        fieldPane.repaint();
    }

    private void _getItemData(CMBItem item) {
        int nAttr = 0;

        try {
            String[] attrArray = item.getAttrNames();
            String[] valueArray = item.getAttrValues();
            nAttr = attrArray.length;
            valList = new Vector(nAttr);
            attrList = new Vector(nAttr);
            for (int i = 0; i < nAttr; i++) {
                valList.addElement(valueArray[i]);
                attrList.addElement(attrArray[i]);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    //*****************************************************************************
    // initGUI
    //*****************************************************************************
    private void initGUI(CMBItem item) {

        frame = new JFrame("Item Attribute Editor");
        mainPane = new CMBBasePanel();

        Border raisedBorder = new SoftBevelBorder(BevelBorder.RAISED);
        Border loweredBorder = new SoftBevelBorder(BevelBorder.LOWERED);

        //*****************************************************************************
        // Widgets creation and event listener setup
        //*****************************************************************************
        // Center panel
        //*****************************************************************************
        fieldPane = new JPanel() {
            public Insets getInsets() {
                return new Insets(20, 20, 20, 20);
            }
        };

        if (item != null) {
            _getItemData(item);
            _setupAttrPane();
        }

        //*****************************************************************************
        // Bottom panel
        //*****************************************************************************
        okButton = new JButton("OK");
        cancelButton = new JButton("Cancel") {
            public Insets getInsets() {
                return new Insets(6, 12, 6, 12); // make it spacy
            }
        };

        okButton.setActionCommand(_cmdOK);
        cancelButton.setActionCommand(_cmdCancel);

        CMBActionAdapter al = new CMBActionAdapter() {
            public void actionPerformed(ActionEvent evt) {
                String cmd = evt.getActionCommand();

                if (cmd.equals(_cmdOK)) {
                    postCMBDataRequest();
                    frame.setVisible(false);
                } else if (cmd.equals(_cmdCancel)) {
                    frame.setVisible(false);
                }
                evt = null;
            }
        };
        okButton.addActionListener(al);
        cancelButton.addActionListener(al);

        okButton.setBorder(raisedBorder);
        cancelButton.setBorder(raisedBorder);

        JPanel buttonPane = new JPanel();
        buttonPane.setLayout(new GridLayout(1, 2, 4, 2));
        buttonPane.add(okButton);
        buttonPane.add(cancelButton);

        //*****************************************************************************
        // Add widgets to window frame
        //*****************************************************************************
        mainPane.addCenterPanel(fieldPane);
        mainPane.addBottomPanel(buttonPane);
        frame.getContentPane().add(mainPane, BorderLayout.CENTER);
        frame.pack();
        CMBUtil.setLocation(frame, CMBUtil.LEFT);

        if (item == null) {
            frame.setVisible(false);
            System.out.println("No item, frame is not visible++++++++");
        } else {
            System.out.println("Has item, frame is now visible+++++++++++");
            frame.setVisible(true);
            frame.requestFocus();
        }
    }

    private void postCMBDataRequest() {
        System.out.println("Posting CMBDataRequestEvent ...");
        if (_action == ItemRequestEvent.ATTR_VIEW) {
            return;
        }

        _getUpdatedValues();
        int eventId = 0;
        Object data = null;
        Object extraData = null;
        switch (_action) {
            case ItemRequestEvent.ITEM_COPYCREATE :
                data = _item;
                extraData = _item.getEntityName();
                eventId = CMBDataRequestEvent.CMB_REQUEST_ITEM_CREATE;
                System.out.println("Eventid = CMB_REQUEST_ITEM_CREATE");
                break;
            case ItemRequestEvent.ATTR_UPDATE :
                data = _item;
                eventId = CMBDataRequestEvent.CMB_REQUEST_ITEM_UPDATE;
                System.out.println("Eventid = CMB_REQUEST_ITEM_UPDATE");
                break;
            case ItemRequestEvent.ATTR_REINDEX :
                data = _item;
                extraData = _item.getEntityName();
                eventId = CMBDataRequestEvent.CMB_REQUEST_ITEM_REINDEX;
                System.out.println("Eventid = CMB_REQUEST_ITEM_REINDEX");
                break;
            case ItemRequestEvent.ATTR_UNINDEX :
                data = _item;
                eventId = CMBDataRequestEvent.CMB_REQUEST_ITEM_UNINDEX;
                System.out.println("Eventid = CMB_REQUEST_ITEM_UNINDEX");
                break;
        }

        dataReqEvent = new CMBDataRequestEvent(this, eventId, (CMBItem) data, extraData);

        Vector v;
        synchronized (this) {
            v = (Vector) listenerVector.clone();
        }

        int cnt = v.size();
        CMBDataRequestListener listener = null;
        for (int i = 0; i < cnt; ++i) {
            System.out.println("Posting to CMBDataRequestListener #" + i);
            listener = (CMBDataRequestListener) v.elementAt(i);
            listener.onCMBDataRequest(dataReqEvent);
        }

    }

    private void _getUpdatedValues() {
        if (txtFieldList.size() != attrList.size())
            System.out.println("ERROR - txtFieldList.size() != attrList.size()...");
        int nAttrs = attrList.size();
        for (int i = 0; i < nAttrs; i++) {
            try {
                String attrVal = ((JTextField) txtFieldList.elementAt(i)).getText();
                String attrName = (String) attrList.elementAt(i);
                System.out.println("Setting value = " + attrVal + " for attrName=" + attrName);
                _item.setAttrValue(attrName, attrVal);
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }

    //*****************************************************************************
    // private data
    //*****************************************************************************
    private final String _cmdOK = "ok";
    private final String _cmdCancel = "cancel";

    private VetoableChangeSupport vcSupport = new VetoableChangeSupport(this);
    private PropertyChangeSupport pcSupport = new PropertyChangeSupport(this);

    //*****************************************************************************
    // GUI widgets, all labels are editable properties
    //*****************************************************************************
    private JButton okButton = null;
    private JButton cancelButton = null;

    private JFrame frame = null;
    private JPanel fieldPane = null;
    private CMBBasePanel mainPane = null;
    private CMBItem _item = null;
    private int _action = 0;
    //*****************************************************************************
    // Transient data members
    //*****************************************************************************
    private transient Vector listenerVector = new Vector();
    private transient Vector txtFieldList = new Vector();
    private transient Vector attrList = new Vector();
    private transient Vector valList = new Vector();

    private transient CMBDataRequestEvent dataReqEvent = null;
    //*****************************************************************************
    // VisualCafe 3.0 support 04/23/99/afic
    //*****************************************************************************

    //*****************************************************************************
    // Copyright
    //*****************************************************************************
    private static final String ibmid = "5648-D03 and 5648-D04 " + "(C) Copyright IBM Corp. 1994,1999";

    class TextFieldActionListener implements java.awt.event.ActionListener {
        public TextFieldActionListener(String attrName) {
            _attrName = attrName;
        }

        public void actionPerformed(ActionEvent evt) {
            if (evt.getSource() instanceof JTextField) {
                JTextField txtField = (JTextField) evt.getSource();
                String txtValue = txtField.getText();
                try {
                    System.out.println("Setting value = " + txtValue + " for attrName=" + _attrName);
                    _item.setAttrValue(_attrName, txtValue);
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        }

        private String _attrName = null;
    }

}