/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import  com.ibm.mm.beans.*;

/**
 * Adds an item to a folder.
 */
public class TAddToFolder {

	public static void main(String args[]) throws Exception {
		if (args.length < 5) {
			System.out.print(
				"Adds an item to a folder. \n"
					+ "Syntax: \n"
					+ "  java TAddToFolder <dstype> <server> <userid> <password> <folder_entity> <folder_condition> / <item_entity> <item_condition> \n"
					+ "Examples:"
					+ "  java TAddToFolder DL libsrvrn frnadmin password NOINDEX Userid LIKE 'owings' / NOINDEX Source LIKE 'tiff' \n");
			System.exit(0);
		}

		// Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String folderEntity = args[4];
        String folderCondition = "";
        int i;
        for (i = 5; i < args.length && !args[i].equals("/"); i++) {
            folderCondition += " " + args[i];
        }
		String itemEntity = args[++i];
		String itemCondition = "";
		i++;
		for (;i < args.length; i++) {
			itemCondition += " " + args[i];
		}

        // Create beans.  The CMBSearchResults, CMBDataManagement, and
        // CMBQueryService beans can be obtained from the CMBConnection bean.
        CMBConnection connection = new CMBConnection();
        CMBQueryService queryService = connection.getQueryService();
        CMBSearchResults searchResults = new CMBSearchResults();
        searchResults.setConnection(connection);
        CMBDataManagement dataManagement = connection.getDataManagement();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();

        // Connect (using method from TConnect sample
        TConnect.connect(connection, dstype, server, userid, password);

		// Search for the folder
		TSearch.search(queryService, dstype, folderEntity, folderCondition, false);
		searchResults.newResults(queryService.getResults());
		if (searchResults.getCount() == 0) {
			throw new Exception("No items matched folder entity/condition: "+folderEntity+"/"+folderCondition);
		}
		if (searchResults.getCount() > 1) {
			throw new Exception("More than one item matched folder entity/condition: "+folderEntity+"/"+folderCondition);
		}
		CMBItem folderItem = searchResults.getItem(0);
		if (folderItem.getItemType() != CMBItem.CMB_TYPE_FOLDER) {
			throw new Exception("Item matching folder criteria is not a folder, entity/condition: "+folderEntity+"/"+folderCondition);
		}
		System.out.println("Folder found");
		
		// Search for the items to add to folder
		TSearch.search(queryService, dstype, itemEntity, itemCondition, false);
		searchResults.newResults(queryService.getResults());
		if (searchResults.getCount() == 0) {
			throw new Exception("No items matched item entity/condition: "+itemEntity+"/"+itemCondition);
		}
		System.out.println(searchResults.getCount() + " items found");
		
		// Add the items to the folder
		dataManagement.setDataObject(folderItem);
		if (dataManagement.isSupported("checkOut")) { // some servers require checkout
			dataManagement.checkOut();
		}
		for (int j = 0; j < searchResults.getCount(); j++) {
			CMBItem item = searchResults.getItem(j);
			System.out.println("Adding item " + item.getName() + " to folder");
			folderItem.addFolderItem(item);
		}
		if (dataManagement.isSupported("checkIn")) {
			dataManagement.checkIn();
		}
		System.out.println("Disconnecting from server");
		connection.disconnect();
	}

}