/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

import  com.ibm.mm.beans.*;

/**
 * Creates a Folder.
 */
public class TCreateFolder {
	
	public static void main(String[] args) throws Exception {
        if (args.length < 6) {
        	System.out.println("Syntax: java TCreateFolder <dstype> <server> <userid> <password> <entity> <attrs>");
            System.exit(0);
        }

        // Read args
        String dstype = args[0];
        String server = args[1];
        String userid = args[2];
        String password = args[3];
        String entity = args[4];
        String attrsString = "";
        for (int i = 5; i < args.length; i++) {
            attrsString += " " + args[i];
        }

        // Create beans
        CMBConnection connection = new CMBConnection();
        CMBSchemaManagement schemaManagement = connection.getSchemaManagement();
        CMBDataManagement dataManagement = connection.getDataManagement();

		// Connect to server
        TConnect.connect(connection, dstype, server, userid, password);

		// Create item on server
        createFolder(schemaManagement, dataManagement, entity, attrsString);

		// Disconnect from server
        TConnect.disconnect(connection);

	}
	
    /**
     * Creates a folder on the server.
     * @param schemaManagement an instance of CMBSchemaManagement bean, to get
     *        information on possible attributes
     * @param dataManagement an instance of CMBDataManagement bean, to perform
     *        the item add.
     * @param entity the name of the entity to create the item under
     * @param attrsString the attributes to give the item, in the form name=value
     */
    public static void createFolder(
        CMBSchemaManagement schemaManagement,
        CMBDataManagement dataManagement,
        String entity,
        String attrsString)
        throws CMBException {

		// Construct a CMBItem object to represent the new item
		System.out.println("Creating CMBItem and setting attributes");
		CMBItem item = new CMBItem();
        item.setConnection(dataManagement.getConnection());
		item.setItemType(CMBBaseConstant.CMB_TYPE_FOLDER);
        item.setServerType(dataManagement.getConnection().getDsType());
        item.setServerName(dataManagement.getConnection().getServerName());

        // Look up the entity, to make sure it exists and to get attribute metadata
        CMBEntity entityBean = schemaManagement.getEntity(entity);
        if (entityBean == null) {
            throw new IllegalArgumentException("Entity not found: " + entity);
        }
        item.setEntityName(entity);

		// Set the attributes on the new item
        TCreateItem.setItemAttrs(entityBean, item, attrsString);

		// Create the item on the server
        System.out.println("Creating folder on server.");
        dataManagement.setDataObject(item);
        dataManagement.createItem(entity);
    }

}