/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Semantic Type Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Semantic Type definition because it was not defined.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSemanticTypeDeletionICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Semantic Type Definitions and Sample Semantic Types.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Semantic Type Definitions Using it's del() Method.
 *                    Deleting Semantic Type Definitions Using the Datastore Definition's del() Method.
 *                    Deletes Semantic Types Created by SSemanticTypeCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SSemanticTypeDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Semantic Types should already be defined.  If not, Sample
 *                    SSemanticTypeCreationICM should be run first to create the Semantic
 *                    Types.  This sample will still complete, but only those that exist 
 *                    will be deleted.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SSemanticTypeDeletionICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SSemanticTypeDeletionICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SSemanticTypeDeletionICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //----------------------------------------------------------------
            // Retrieve the Semantic Type Definition
            //----------------------------------------------------------------
            // For more information about the defined Semantic Types, please consult
            // the SSemanticTypeCreationICM sample.
            System.out.println("Retrieving the Semantic Type Definition of 'S_myDocument'...");

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKSemanticTypeDefICM semTypeDef = (DKSemanticTypeDefICM) dsDefICM.retrieveSemanticType("S_myDocument"); // Retrieve the definition.

            System.out.println("Retrieved the Semantic Type Definition of 'S_myDocument'.");

            //-----------------------------------------------------------------------------
            // Delete the Semantic Type Definition in the Datastore Definition.
            //-----------------------------------------------------------------------------
            // The Semantic Type Definition can be deleted using its own del() method or
            // by using the Datastore Definition's del() funciton.
            System.out.println("Deleting the Semantic Type Definition 'S_myDocument' in Datastore Definition...");

                if(semTypeDef!=null) // if it is not already deleted,
                    semTypeDef.del(); // Delete the Semantic Type Definition

                // NOTE:  dsDefICM.del(semTypeDef); could also be used here.

            System.out.println("Deleted the Semantic Type Definition 'S_myDocument' in Datastore Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Semantic Type Definition
    * @param dsICM             Connected DKDatastoreICM object.
    * @param semanticTypeName  Semantic Type Name for the Semantic Type Definition to Delete.
    **/
    public static void deleteSemanticTypeDef(DKDatastoreICM dsICM, String semanticTypeName) throws DKException, Exception{
        System.out.println("Deleting the Semantic Type Definition '"+semanticTypeName+"'...");

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
            DKSemanticTypeDefICM semTypeDef = (DKSemanticTypeDefICM) dsDefICM.retrieveSemanticType(semanticTypeName); // Retrieve the definition.
            if(semTypeDef!=null)    // if it is not already deleted...
                semTypeDef.del();   // Delete the Semantic Type Definition

        System.out.println("Deleted the Semantic Type Definition '"+semanticTypeName+"'.");
    }
    
}//end class SSemanticTypeDeletionICM
              
