/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Not a sample program.  This file is a tool provided as-is.

For information on creating Attribute Definitions, please refer to the official samples, such
as:
    - SAttributeDefinitionCreationICM
    - SAttributeDefinitionRetrievalICM
    - SAttributeGroupDefCreationICM
    - SAttributeGroupDefRetrievalICM
    - SReferenceAttrDefCreationICM
    - SReferenceAttrDefRetrieveICM
 
Tool for Attribute Definition Import from XML.
    This tool will import a Attribute & Attribute Group Definitions described in an
    XML file that follows the TXMLAttrDefImportICM.dtd.
    
    Example Definitions  File
    -------------------  --------------------------------
    All Needed by Item   TXMLAttrDefImportICM_Example.xml
    Type XML Examples   
    
    Examlple Usage:
        java TXMLAttrDefImportICM -f TXMLAttrDefImportICM_Example.xml

********************************************************************/

import org.apache.xerces.parsers.DOMParser;
import org.apache.xerces.*;
import org.w3c.dom.*;
import org.xml.sax.*;

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;

import java.util.Vector;
import java.sql.Date;
import java.io.IOException;
import java.lang.*;

/************************************************************************************************
 *          FILENAME: TXMLAttrDefImportICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Tool that imports an Attribute Definition from an XML file that follows the
 *                    TXMLAttrDefImportICM.dtd.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Not a sample program.  This file is a tool provided as-is.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java TXMLAttrDefImportICM -f <xml file path> -u <username> 
 *                                              -d <database> -p <password>
 *                    NOTE:  xml file name required.
 *                    NOTE:  xml file must follow the TXMLAttrDefImportICM.dtd
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Classpath must contain file location of xerces.jar.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 *                    SAttributeDefinitionRetrievalICM
 *                    TXMLInternalValidatingParserICM
 *                    TXMLAttrDefImportICM.dtd
 ************************************************************************************************/
class TXMLAttrDefImportICM{

    // Public Constants for Defaults Used by Samples / Tools
    public static String DEFAULT_DATABASE = "icmnlsdb";
    public static String DEFAULT_USERNAME = "icmadmin";
    public static String DEFAULT_PASSWORD = "password";

    // Constants
	static final short NODE_TEXT = 3;
	static final int   ADD_ATTR  = 1;

    // Variables
	Document          _doc;
	DKDatastoreICM    _ds;
	DKDatastoreDefICM _dsDefICM;
	boolean           isAttrGroup = false;
	DKAttrGroupDefICM attrGroup;

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Tool.
     * @param argv[] String Array containing arguments.  Optional arguments are -d <databse> -u <userName> -p <password>.
     */
	public static void main(String[] argv) throws DKException, Exception{
		
		TXMLAttrDefImportICM createAttr         = null; 
		TXMLInternalValidatingParserICM vParser = null;

		//*** for input parameters
		String database = DEFAULT_DATABASE;
		String userName = DEFAULT_USERNAME;
		String password = DEFAULT_PASSWORD;
		String file     = "FILE_NOT_SPECIFIED";

		int    length   = argv.length;
		String param    = null;

		//--------------------------------------------------------------
		// Checking for input parameters
		//--------------------------------------------------------------
		if (argv.length < 7) {
			System.out.println("Usage: " );
			System.out.println("  java TXMLAttrDefImportICM -f <xml file path> -u <username>" );
            System.out.println("                            -d <database> -p <password>");
            System.out.println("  NOTE:  xml file name required.");
            System.out.println("  NOTE:  xml file must follow the TXMLAttrDefImportICM.dtd");
			System.out.println("  *** Some parms not specified, using defaults..." );
			System.out.println("");
		}
		for(int i=0; i<length; i++){
			param = argv[i];
			if(param.startsWith("-f"))
				file = argv[++i];
			else if(param.startsWith("-u"))
				userName = argv[++i];
			else if(param.startsWith("-p"))
				password = argv[++i];
			else if(param.startsWith("-d"))
				database = argv[++i];
	    }//end for 
        String ver = SConnectDisconnectICM.EIP_VERSION;

        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Tool Program:  TXMLAttrDefImportICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println(" FileName: "+file);
        System.out.println("===========================================\n");

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Run Tool
            //-------------------------------------------------------------
            System.out.println("Running Tool...");

            createAttr = new TXMLAttrDefImportICM(dsICM);
            vParser    = new TXMLInternalValidatingParserICM();

            createAttr._doc = vParser.open(file); //open and validate xml file

            Element root_attrs = (Element)createAttr._doc.getDocumentElement();

			createAttr.traverseAttrs(root_attrs); //begin to create attributes and/or attribute groups

            System.out.println("Tool Completed.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Tool program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Tool program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function from the connection sample.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function from the connection sample.
            throw(exc);
        }
    }//end main
	
   /**
    * Sets datastore and datastoredef from main and sets _doc to null
    * @param ds datastore
    * @param dsDefICM   datastoredef
    **/
	TXMLAttrDefImportICM(DKDatastoreICM dsICM) throws DKException, Exception{
		_doc = null;
		_ds = dsICM;
		_dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the Datastore Definition Object
	}

   /**
    * Traverses children of the attribute node and creates attributes and sets name and description
    * @param node   an attribute node in the xml document
    **/     
	private void traverseAttrs(Node node) throws DKException, Exception {
		DKAttrDefICM attr;
		boolean attrExists = false;
		boolean attrGroupExists = false;
		DKAttrGroupDefICM refAttrGroup;

		NodeList children = node.getChildNodes();
		NamedNodeMap attrs;
		Node n;
		int size;

		if ((children != null) ) {
			for (int i=0; i< children.getLength(); i++) //traversing attributes of itemtype
			{
				String attrname = null;
				String attrGroupName = null;
				String node_type = children.item(i).getNodeName();

				int type = children.item(i).getNodeType();

				if ((type == Node.ELEMENT_NODE)&&(node_type.equals("attrGroup"))){//define attribute group
					isAttrGroup = true;
					attrGroup = new DKAttrGroupDefICM(_ds);
					attrs = children.item(i).getAttributes();

					if (attrs != null) {
						size = attrs.getLength();
						for (int j = 0; j < size; j++) {//setting attributes of attrs
							n = attrs.item(j);

							if(n.getNodeName().equals("name")){
								attrGroupName = getValue(n);

								if(getAttrGroup(attrGroupName)!=null){
									attrGroupExists = true;
									System.out.println("---------------------------------");
									System.out.println("Attribute Group "+attrGroupName+" Already Exists.");
								}
								else{
								    attrGroupExists = false;
									System.out.println("---------------------------------");
									System.out.println("Creating Attribute Group "+attrGroupName+"...");
									attrGroup.setName(attrGroupName);
								}
							}
							else if(n.getNodeName().equals("description"))
							{
								attrGroup.setDescription(getValue(n));
							} 
						}//for loop

						if(!attrGroupExists){
							traverseAttrs(children.item(i));//add attributes to attrGroup
							isAttrGroup = false;
							attrGroupExists = false;
							System.out.println("---------------------------------");
							System.out.println("Adding Attribute Group with Above Attribute Members");
							attrGroup.add();//add to LS
							System.out.println("Complete.");
						}		        
					}//if attr!=NULL       
				}
				else if ((type == Node.ELEMENT_NODE)&&(node_type.equals("refAttrGroup"))){//define attribute group
					refAttrGroup = new DKAttrGroupDefICM(_ds, DKConstantICM.DK_ICM_REFERENCE_ATTRGROUP);
					attrs = children.item(i).getAttributes();
                    isAttrGroup = false;
                    
					if (attrs != null) {
						size = attrs.getLength();
						for (int j = 0; j < size; j++) {//setting attributes of attrs
							n = attrs.item(j);

							if(n.getNodeName().equals("name")){
								attrGroupName = getValue(n);

								if(getAttrGroup(attrGroupName)!=null){
									attrGroupExists = true;
									System.out.println("---------------------------------");
									System.out.println("Reference Attribute Group "+attrGroupName+" Already Exists.");
								}
								else{
								    attrGroupExists = false;
									System.out.println("---------------------------------");
									System.out.println("Creating Reference Attribute Group "+attrGroupName+"...");
									refAttrGroup.setName(attrGroupName);
								}
							}
							else if(n.getNodeName().equals("description"))
							{
								refAttrGroup.setDescription(getValue(n));
							} 
						}//for loop

						if(!attrGroupExists){
							refAttrGroup.add();//add to LS
							System.out.println("Complete.");
						}		        
					}//if attr!=NULL       
				}
				else if ((type == Node.ELEMENT_NODE)&&(node_type.equals("attribute"))){
					attr = new DKAttrDefICM(_ds);
					attrs = children.item(i).getAttributes();

					if (attrs != null) {
						size = attrs.getLength();
						for (int j = 0; j < size; j++) {//setting attributes of attrs
							n = attrs.item(j);

							if(n.getNodeName().equals("name")){
								attrname = getValue(n);

								if(getAttr(attrname)!=null){
									attrExists = true;
									System.out.println("---------------------------------");
									System.out.println("Attribute "+attrname+" Already Exists.");
								}
								else{
								    attrExists=false;
									System.out.println("---------------------------------");
									System.out.println("Creating Attribute "+attrname+"...");
									attr.setName(attrname);
								}
							}
							else if(n.getNodeName().equals("description"))
							{
								attr.setDescription(getValue(n));
							} 
						}//for loop
					}//if attr!=NULL
					if(!attrExists){
						addAttrTypeInfo(attr, children.item(i));
						SAttributeDefinitionRetrievalICM.printAttrDef(_ds,attrname);
						attrExists = false;
					}

				}//if type == element
			}//forloop
			if(!isAttrGroup)
				System.out.println("---------------------------------");
		}//if children !=NULL
	}

   /**
    * Traverses the attribute node and sets type.  Possible types are : timestamp, date, char, varchar, short, integer, long, float, decimal, double, time, blob, clob, dbclob. 
    *
    * @param node   an attribute type node in the xml document
    * @param attr   Pointer to attribute from caller
    * 
    */
    public void addAttrTypeInfo(DKAttrDefICM attr, Node node) throws DKException, Exception{
		boolean more = false;

		NodeList children = node.getChildNodes();
		if ((children != null)) {
			for (int i=0; i< children.getLength(); i++) 
			{
				int childrenType = children.item(i).getNodeType();

				if ((childrenType == Node.ELEMENT_NODE)){

					String type = children.item(i).getNodeName();

					if (type.equals("timestamp"))
					{
						attr.setType(DKConstant.DK_CM_TIMESTAMP);
					}
					else if (type.equals("date"))
					{
						attr.setType(DKConstant.DK_CM_DATE);
					}
					else if (type.equals("char"))
					{
						attr.setType(DKConstant.DK_CM_CHAR);
						more = true;
					}
					else if (type.equals("varchar"))
					{
						attr.setType(DKConstant.DK_CM_VARCHAR);
						more = true;
					}
					else if (type.equals("short"))
					{
						attr.setType(DKConstant.DK_CM_SHORT);
					}
					else if (type.equals("integer"))
					{
						attr.setType(DKConstant.DK_CM_INTEGER);
					}
					else if (type.equals("long"))
					{
						attr.setType(DKConstant.DK_CM_LONG);
					}
					else if (type.equals("float"))
					{
						attr.setType(DKConstant.DK_CM_FLOAT);
					}
					else if (type.equals("decimal"))
					{
						attr.setType(DKConstant.DK_CM_DECIMAL);
						more = true;
					}
					else if (type.equals("double"))
					{
						attr.setType(DKConstant.DK_CM_DOUBLE);
					}
					else if (type.equals("time"))
					{
						attr.setType(DKConstant.DK_CM_TIME);
					}
					else if (type.equals("blob"))
					{
						attr.setType(DKConstant.DK_CM_BLOB);
						more = true;
					}
					else if (type.equals("clob"))
					{
						attr.setType(DKConstant.DK_CM_CLOB);
						more = true;
					}
					else if (type.equals("dbclob"))
					{
						attr.setType(DKConstant.DK_CM_DBCLOB);
						more = true;
					}

					if(more){
						addMoreAttrTypeInfo(children.item(i), attr);
					}
					else{
						attr.add();
						if(isAttrGroup){
							attrGroup.addAttr(attr);  
						}
					}

				}
			}
		}
	}

   /**
    * Sets attributes of the attribute type node.  Possible attributes of attributes are: float-precision, scale; varchar-min, max; clob-unit.
    *
    * @param node   an attribute type node in the xml document
    * @param attr   attribute pointer from caller
    * 
    */
    public void addMoreAttrTypeInfo(Node node, DKAttrDefICM attr) throws DKException, Exception{

		NamedNodeMap attrs = node.getAttributes();

		int clob_size=0;
		int unit =1;

		if (attrs != null) {
			int size = attrs.getLength();
			for (int j = 0; j < size; j++) {//setting attributes of attrs
				Node n = attrs.item(j);

				if(n.getNodeName().equals("size") || n.getNodeName().equals("len")){
					attr.setSize((new Integer(getValue(n))).intValue());
					clob_size = (new Integer(getValue(n))).intValue();
				}
				else if(n.getNodeName().equals("precision"))
				{
					attr.setSize((new Integer(getValue(n))).intValue());
				} 
				else if(n.getNodeName().equals("scale"))
				{
					attr.setScale((new Integer(getValue(n))).intValue());
				} 
				else if(n.getNodeName().equals("min"))
				{
					attr.setMin((new Short(getValue(n))).shortValue());
				}  
				else if(n.getNodeName().equals("max"))
				{
					attr.setMax((new Short(getValue(n))).shortValue());
				} 
				else if(n.getNodeName().equals("unit"))
				{
					if(getValue(n).equals("GB"))
					{
						unit = 10000;
					}
					else if(getValue(n).equals("MB"))
					{
						unit=1000;
					}

					if(clob_size>0){
						attr.setSize(clob_size * unit); 
					}
				}
			}//for loop
		}//if attr!=NULL   
		attr.add();
		if(isAttrGroup){
			attrGroup.addAttr(attr);      
		}
	}

   /**
    * Obtain an attribute.
    * NOTE:  Please refer to SAttributeDefinitionRetrievalICM for official documentation
    *        and funcitons like this.
    * @param name   Name of attribute
    * @return  Returns the requested attribute definition.
    **/
    public DKAttrDefICM getAttr(String name) throws DKException, Exception {
		return((DKAttrDefICM)_dsDefICM.retrieveAttr(name));
	}

   /**
    * Obtain the Attribute Group with the given name..
    * NOTE:  Please refer to SAttributeDefinitionRetrievalICM for official documentation
    *        and funcitons like this.
    * @param name   Name of Attribute Group.
    * @return  Returns the requested Attribute Group Definition.
    **/
    public DKAttrGroupDefICM getAttrGroup(String name) throws DKException, Exception {
		return((DKAttrGroupDefICM)_dsDefICM.retrieveAttrGroup(name));
    }

   /**
    * Method to get text from node. Get the first subnode from the
    * node and extract nodeValue from it. Text is usually the first child value
    * in a node. 
    * @param n  Node that value is wanted to be extracted from
    **/
    private String getValue(Node n) {
		String result = null;
		if (n.hasChildNodes()) {
			Node firstchild = n.getFirstChild();
			if (firstchild.getNodeType() == NODE_TEXT)
				result = firstchild.getNodeValue();
		}

		if (result != null) {
			result = result.trim();

			if (!result.equals(""))
				return(result);
		} 

		return(null);
	}

}//end TXMLAttrDefImportICM


